@@ -1,4 +1,4 @@
-/* @(#) $Id: actsync.c,v 1.22 1996/11/29 15:14:01 brister Exp $ */
+/* @(#) $Id: actsync.c,v 1.23 1996/12/16 20:51:30 brister Exp $ */
 /* @(#) Under RCS control in /usr/local/news/src/inn/local/RCS/actsync.c,v */
 /*
  * actsync - sync or merge two active files
@@ -387,7 +387,7 @@ struct eqgrp {
  * path to ctlinnd command
  */
 /* =()<#define CTLINND_PATH "@<_PATH_NEWSBIN>@/ctlinnd">()= */
-#define CTLINND_PATH "/usr/news/bin/ctlinnd"
+#define CTLINND_PATH "/var/spool/news/bin/ctlinnd"
 
 #define DEV_NULL "/dev/null"	/* path to the bit bucket */
 #define CTLINND_NAME "ctlinnd"	/* basename of ctlinnd command */
@@ -3117,11 +3117,11 @@ exec_cmd(mode, cmd, grp, type, who)
     char buf[BUFSIZ+1];		/* interactive buffer */
     int pid;			/* pid of child process */
     int io[2];			/* pair of pipe descriptors */
-#if defined(UNION_WAIT)
+#if defined(DO_USE_UNION_WAIT)
     union wait status;		/* wait status */
 #else
     int status;			/* wait status */
-#endif /* defined(UNION_WAIT) */
+#endif /* defined(DO_USE_UNION_WAIT) */
     int exitval;		/* exit status of the child */
     char *p;
 
