@@ -1,4 +1,4 @@
-/*  $Revision: 1.38 $
+/*  $Revision: 1.40 $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -990,15 +990,16 @@ ValidNewsgroups(hdr, F, article)
 	case NF_FLAG_OK:
 	    break;
 	case NF_FLAG_MODERATED:
-	    if (Dump)
-		(void)fprintf(stderr,
-			"%s is moderated -- article would be mailed\n",
-			group);
-	    else if (!approved) {
-		mailed = MailArticle(group, article);
-		DISPOSE(DDend(h));
-		QuitServer(mailed ? 0 : 1);
-	    }
+	    if (!approved)
+		if (Dump)
+		    (void)fprintf(stderr,
+				"%s is moderated -- article would be mailed\n",
+				  group);
+		else {
+		    mailed = MailArticle(group, article);
+		    DISPOSE(DDend(h));
+		    QuitServer(mailed ? 0 : 1);
+		}
 	    break;
 	case NF_FLAG_IGNORE:
 	case NF_FLAG_NOLOCAL:
@@ -1331,7 +1332,11 @@ main(ac, av)
 	}
 
     /* Check the newsgroups. */
-    if ((F = CAlistopen(FromServer, ToServer, (char *)NULL)) != NULL) {
+    if ((F = CAlistopen(FromServer, ToServer, (char *)NULL)) == NULL) {
+	if (NNTPsendpassword((char *)NULL, FromServer, ToServer) >= 0)
+	    F = CAlistopen(FromServer, ToServer, (char *)NULL);
+    }
+    if (F != NULL) {
 	if (!ValidNewsgroups(HDR(_newsgroups), F, article)) {
 	    CAclose();
 	    QuitServer(1);
