@@ -1,14 +1,15 @@
-/*  $Revision: 1.1 $
+/*  $Revision: 1.31 $
 **
 **  A front-end for InterNetNews.
 **  Read UUCP batches and offer them up NNTP-style.  Because we may end
 **  up sending our input down a pipe to uncompress, we have to be careful
 **  to do unbuffered reads.
 */
-#include "configdata.h"
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
+#include <ctype.h>
 #if	defined(DO_NEED_TIME)
 #include <time.h>
 #endif	/* defined(DO_NEED_TIME) */
@@ -20,7 +21,6 @@
 #include "paths.h"
 #include "logging.h"
 #include "libinn.h"
-#include "clibrary.h"
 #include "mydir.h"
 #include "macros.h"
 
@@ -38,7 +38,7 @@ STATIC char	SPOOLNEWS[] = _PATH_SPOOLNEWS;
 STATIC char	SPOOLTEMP[] = _PATH_SPOOLTEMP;
 STATIC FILE	*FromServer;
 STATIC FILE	*ToServer;
-STATIC char	UNPACK[] = "news-unpack";
+STATIC char	UNPACK[] = "compress";
 STATIC HEADER	RequiredHeaders[] = {
     { "Message-ID",	10 },
 #define _messageid	0
@@ -85,7 +85,7 @@ StartChild(fd, path, argv)
 {
     int		pan[2];
     int		i;
-    int		pid;
+    PID_T	pid;
 
     /* Create a pipe. */
     if (pipe(pan) < 0) {
@@ -144,7 +144,7 @@ STATIC void
 WaitForChildren(i)
     register int	i;
 {
-    register int	pid;
+    register PID_T	pid;
     int			status;
 
     while (--i >= 0) {
@@ -160,21 +160,6 @@ WaitForChildren(i)
 
 
 
-/*
-**  Make a temporary filename that is unlikely to collide with mktemp
-**  or cause problems for sites with 14-character filename limits,
-**  and that hints at the sending host.
-*/
-STATIC void
-TempName(dir, buff)
-    char	*dir;
-    char	*buff;
-{
-    (void)sprintf(buff, "%s/%.8sXXXXXX", dir, UUCPHost ? UUCPHost : "unknown");
-    (void)mktemp(buff);
-}
-
-
 /*
 **  Clean up the NNTP escapes from a line.
 */
@@ -203,9 +188,11 @@ Reject(article, reason, arg)
     char	*reason;
     char	*arg;
 {
+#if	defined(DO_RNEWS_SAVE_BAD)
     char	buff[SMBUF];
     FILE	*F;
     int		i;
+#endif	/* defined(DO_RNEWS_SAVE_BAD) */
 
     syslog(L_NOTICE, reason, arg);
     if (Verbose) {
@@ -214,6 +201,7 @@ Reject(article, reason, arg)
 	(void)fprintf(stderr, " [%.40s...]\n", article);
     }
 
+#if	defined(DO_RNEWS_SAVE_BAD)
     TempName(_PATH_BADNEWS, buff);
     if ((F = fopen(buff, "w")) == NULL) {
 	syslog(L_ERROR, "cant fopen %s %m", buff);
@@ -224,6 +212,7 @@ Reject(article, reason, arg)
 	syslog(L_ERROR, "cant fwrite %s %m", buff);
     if (fclose(F) == EOF)
 	syslog(L_ERROR, "cant close %s %m", buff);
+#endif	/* defined(DO_RNEWS_SAVE_BAD) */
 }
 
 
@@ -255,7 +244,7 @@ Process(article)
     for (hp = RequiredHeaders; hp < ENDOF(RequiredHeaders); hp++) {
 	if ((p = HeaderFind(article, hp->Name, hp->size)) == NULL) {
 	    Reject(article, "bad_article missing %s", hp->Name);
-	    return TRUE;
+	    return FALSE;
 	}
 	if (IS_MESGID(hp)) {
 	    id = p;
@@ -274,7 +263,7 @@ Process(article)
     /* Send the NNTP "ihave" message. */
     if ((p = strchr(id, '\n')) == NULL) {
 	Reject(article, "bad_article unterminated %s header", "Message-ID");
-	return TRUE;
+	return FALSE;
     }
     *p = '\0';
     (void)fprintf(ToServer, "ihave %s\r\n", id);
@@ -307,7 +296,7 @@ Process(article)
 #if	defined(FILE_RNEWS_LOG_DUPS)
 	if ((F = fopen(_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
 	    *p = '\0';
-	    (void)fprintf(stderr, "duplicate %s %s", id, path);
+	    (void)fprintf(F, "duplicate %s %s", id, path);
 	    (void)fclose(F);
 	}
 #endif	/* defined(FILE_RNEWS_LOG_DUPS) */
@@ -325,8 +314,7 @@ Process(article)
 	syslog(L_ERROR, "cant fgets after article %m");
 	return FALSE;
     }
-    if ((p = strchr(buff, '\r')) != NULL || (p = strchr(buff, '\n')) != NULL)
-	*p = '\0';
+    (void)REMclean(buff);
     if (!CTYPE(isdigit, buff[0])) {
 	syslog(L_NOTICE, "bad_reply after article %s", buff);
 	return FALSE;
@@ -340,11 +328,7 @@ Process(article)
     case NNTP_TOOKIT_VAL:
 	break;
     case NNTP_REJECTIT_VAL:
-#if	defined(DO_RNEWS_SAVE_BAD)
 	Reject(article, "rejected %s", buff);
-#else
-	syslog(L_NOTICE, "rejected %s", buff);
-#endif	/* defined(DO_RNEWS_SAVE_BAD) */
 	break;
     }
     return TRUE;
@@ -503,6 +487,7 @@ UnpackOne(fdp, countp)
     STRING	cargv[4];
     int		artsize;
     int		i;
+    int		gzip = 0;
     BOOL	HadCount;
     BOOL	SawCunbatch;
 
@@ -516,7 +501,9 @@ UnpackOne(fdp, countp)
 	if (i == 0)
 	    break;
 
-	if (buff[0] != RNEWS_MAGIC1)
+	if (buff[0] == 0x1f)
+	    gzip = 1;
+	else if (buff[0] != RNEWS_MAGIC1)
 	    /* Not a batch file.  If we already got one count, the batch
 	     * is corrupted, else read rest of input as an article. */
 	    return HadCount ? FALSE : ReadRemainder(*fdp, buff[0], '\0');
@@ -531,6 +518,19 @@ UnpackOne(fdp, countp)
 	    return FALSE;
 
 	/* Check second magic character. */
+	/* gzipped ($1f$8b) or compressed ($1f$9d) */
+	if (gzip && ((buff[1] == (char)0x8b) || (buff[1] == (char)0x9d))) {
+	    cargv[0] = "gzip";
+	    cargv[1] = "-d";
+	    cargv[2] = NULL;
+	    lseek(*fdp, (OFFSET_T) 0, 0); /* Back to the beginning */
+	    *fdp = StartChild(*fdp, _PATH_GZIP, cargv);
+	    if (*fdp < 0)
+	        return FALSE;
+	    (*countp)++;
+	    SawCunbatch = TRUE;
+	    continue;
+	}
 	if (buff[1] != RNEWS_MAGIC2)
 	    return HadCount ? FALSE : ReadRemainder(*fdp, buff[0], buff[1]);
 
@@ -613,7 +613,6 @@ Unspool()
     char		buff[SMBUF];
     char		hostname[10];
     int			fd;
-    int			oerrno;
     int			i;
 
     /* Go to the spool directory, get ready to scan it. */
@@ -649,21 +648,23 @@ Unspool()
 	    }
 	    continue;
 	}
-	(void)strncpy(hostname, InputFile, 8);
-	hostname[8] = '\0';
+	/* Get UUCP host from spool file, deleting the mktemp XXXXXX suffix. */
+	if ((i = strlen(InputFile)) > 6)
+	    i -= 6;
+	if (i > sizeof hostname - 1)
+	    /* Just in case someone wrote their own spooled file. */
+	    i = sizeof hostname - 1;
+	(void)strncpy(hostname, InputFile, i);
+	hostname[i] = '\0';
 	UUCPHost = hostname;
 	ok = UnpackOne(&fd, &i);
 	(void)close(fd);
 	WaitForChildren(i);
 
 	if (!ok) {
-	    oerrno = errno;
 	    TempName(_PATH_BADNEWS, buff);
-	    xperror("Unspooling failed");
-	    (void)fprintf(stderr, "Unspooling failed saving to %s %s\n",
-		buff, strerror(errno));
-	    errno = oerrno;
-	    syslog(L_ERROR, "cant unspool saving to %s %m", buff);
+	    (void)fprintf(stderr, "Unspooling failed saving to %s\n", buff);
+	    syslog(L_ERROR, "cant unspool saving to %s", buff);
 	    if (rename(InputFile, buff) < 0) {
 		xperror(buff);
 		syslog(L_FATAL, "cant rename %s to %s %m", InputFile, buff);
@@ -686,8 +687,9 @@ Unspool()
 **  an alternate filesystem?
 */
 STATIC void
-Spool(fd)
+Spool(fd, mode)
     register int	fd;
+    int			mode;
 {
     register int	spfd;
     register int	i;
@@ -698,6 +700,8 @@ Spool(fd)
     int			count;
     int			status;
 
+    if(mode == 'N')
+	exit(9);
     TempName(SPOOLTEMP, temp);
     (void)umask(0);
     if ((spfd = open(temp, O_WRONLY | O_CREAT, BATCHFILE_MODE)) < 0) {
@@ -760,8 +764,10 @@ OpenRemote(server, buff)
 
     *buff = '\0';
     if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0) {
+	int oerrno = errno;
 	(void)fclose(FromServer);
 	(void)fclose(ToServer);
+	errno = oerrno;
 	return FALSE;
     }
     return TRUE;
@@ -782,7 +788,7 @@ CantConnect(buff, mode, fd)
     else
 	syslog(L_FATAL, "cant open_remote %m");
     if (mode != 'U')
-	Spool(fd);
+	Spool(fd, mode);
     exit(1);
 }
 
@@ -811,21 +817,24 @@ main(ac, av)
 
     /* First thing, set up logging and our identity. */
     openlog("rnews", L_OPENLOG_FLAGS, LOG_INN_PROG);
+#if !defined(__FreeBSD__) && !defined(__bsdi__)
     if (setgid(getegid()) < 0) {
-	syslog(L_ERROR, "cant setgid to %d %m", getegid());
+	syslog(L_FATAL, "cant setgid to %d %m", getegid());
+	exit(1);
     }
     if (setuid(geteuid()) < 0) {
 	syslog(L_FATAL, "cant setuid to %d %m", geteuid());
 	exit(1);
     }
-    UUCPHost = getenv(_ENV_UUCPHOST);
+#endif
+     UUCPHost = getenv(_ENV_UUCPHOST);
     (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
     fd = STDIN;
     mode = '\0';
     Slave = NULL;
-    while ((i = getopt(ac, av, "h:S:Uv")) != EOF)
+    while ((i = getopt(ac, av, "h:S:NUv")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -836,6 +845,7 @@ main(ac, av)
 	case 'S':
 	    Slave = optarg;
 	    break;
+	case 'N':
 	case 'U':
 	    mode = i;
 	    break;
@@ -892,7 +902,7 @@ main(ac, av)
 	Unspool();
     else {
 	if (!UnpackOne(&fd, &i))
-	    Spool(fd);
+	    Spool(fd, mode);
 	WaitForChildren(i);
     }
 
