@@ -1,4 +1,4 @@
-/*  $Revision: 1.26 $
+/*  $Revision: 1.27 $
 **
 **  Here be declarations of routines and variables in the C library.
 **  You must #include <sys/types.h> and <stdio.h> before this file.
@@ -46,7 +46,7 @@ typedef int INT32_T;
 #if	defined(SIG_DFL)
     /* What types of variables can be modified in a signal handler? */
     /* =()<typedef @<SIGVAR>@ SIGVAR;>()= */
-typedef sig_atomic_t SIGVAR;
+typedef int SIGVAR;
 #endif	/* defined(SIG_DFL) */
 
 /* =()<#include @<STR_HEADER>@>()= */
@@ -116,14 +116,11 @@ extern FREEVAL		free();
 extern POINTER		malloc();
 extern POINTER		realloc();
 
-#if	defined(ACT_MMAP)
-#define MMAP
-#endif /* defined(ACT_MMAP) */
-#if	defined(MMAP)
+#if	defined (ACT_MMAP) || defined(MMAP)
 /* =()<typedef @<MMAP_PTR>@ MMAP_PTR;>()= */
 typedef caddr_t MMAP_PTR;
 extern MMAP_PTR mmap();
-#endif /* defined(MMAP) */	
+#endif /* defined (ACT_MMAP) || defined(MMAP) */	
 
 /* Some backward systems need this. */
 extern FILE	*popen();
@@ -135,7 +132,7 @@ extern int	strncasecmp();
 /* =()<extern @<ABORTVAL>@	abort();>()= */
 extern void	abort();
 /* =()<extern @<ALARMVAL>@	alarm();>()= */
-extern unsigned int	alarm();
+extern unsigned	alarm();
 /* =()<extern @<EXITVAL>@	exit();>()= */
 extern void	exit();
 /* =()<extern @<GETPIDVAL>@	getpid();>()= */
@@ -145,7 +142,7 @@ extern off_t	lseek();
 /* =()<extern @<QSORTVAL>@	qsort();>()= */
 extern void	qsort();
 /* =()<extern @<SLEEPVAL>@	sleep();>()= */
-extern unsigned int	sleep();
+extern unsigned	sleep();
 /* =()<extern @<_EXITVAL>@	_exit();>()= */
 extern void	_exit();
 
