@@ -1,15 +1,17 @@
-/*  $Revision: 1.29 $
+/*  $Revision: 1.43 $
 **
 **  Here be configuration data used by various InterNetNews programs.
 **  The numbers refer to sections in the config.dist file.
 */
 
+#if ! defined (__configdata_h__)
+#define __configdata_h__ 1
 
 /*
 **  1.  MAKE CONFIG PARAMETERS
 */
     /* =()<#define @<USE_CHAR_CONST>@_USE_CHAR_CONST>()= */
-#define DUNNO_USE_CHAR_CONST
+#define DO_USE_CHAR_CONST
 #if	defined(DO_USE_CHAR_CONST)
 typedef char const	*STRING;
 typedef char * const	CSTRING;
@@ -33,7 +35,7 @@ typedef char		*CSTRING;
 */
 #if	defined(__dead)
     /* BSD4.4 */
-#define NORETURN	__dead
+#define NORETURN	__dead void
 #else
 #if	defined(__GNUC__)
     /* GCC */
@@ -53,7 +55,7 @@ typedef char		*CSTRING;
     /* =()<#define NEWSGID		"@<NEWSGROUP>@">()= */
 #define NEWSGID		"news"
     /* =()<#define NEWSMASTER		"@<NEWSMASTER>@">()= */
-#define NEWSMASTER		"usenet"
+#define NEWSMASTER		"news"
     /* =()<#define PATHMASTER		"@<PATHMASTER>@">()= */
 #define PATHMASTER		"not-for-mail"
     /* Umask to set. */
@@ -75,13 +77,13 @@ typedef char		*CSTRING;
 */
     /* Use have stdargs, varargs, or neither? */
     /* =()<#define VAR_@<VAR_STYLE>@>()= */
-#define VAR_VARARGS
+#define VAR_STDARGS
     /* Use BSD4.2 or Posix directory names? */
     /* =()<#define DIR_@<DIR_STYLE>@>()= */
 #define DIR_DIRENT
     /* Use flock, lockf, or nothing to lock files? */
     /* =()<#define LOCK_@<LOCK_STYLE>@>()= */
-#define LOCK_FLOCK
+#define LOCK_LOCKF
     /* Do you have <unistd.h>? */
     /* =()<#define @<HAVE_UNISTD>@_HAVE_UNISTD>()= */
 #define DO_HAVE_UNISTD
@@ -94,12 +96,15 @@ typedef char		*CSTRING;
     /* Do you have fchmod? */
     /* =()<#define @<HAVE_FCHMOD>@_HAVE_FCHMOD>()= */
 #define DO_HAVE_FCHMOD
+    /* Do you have setproctitle()? */
+    /* =()<#define @<HAVE_SETPROCTITLE>@_HAVE_SETPROCTITLE>()= */
+#define DONT_HAVE_SETPROCTITLE
     /* Do you have setsid()? */
     /* =()<#define @<HAVE_SETSID>@_HAVE_SETSID>()= */
 #define DO_HAVE_SETSID
     /* Does your (struct tm) have a tm_gmtoff field? */
     /* =()<#define @<HAVE_TM_GMTOFF>@_HAVE_TM_GMTOFF>()= */
-#define DO_HAVE_TM_GMTOFF
+#define DONT_HAVE_TM_GMTOFF
     /* Does your (struct stat) have a st_blksize field? */
     /* =()<#define @<HAVE_ST_BLKSIZE>@_HAVE_ST_BLKSIZE>()= */
 #define DO_HAVE_ST_BLKSIZE
@@ -108,7 +113,7 @@ typedef char		*CSTRING;
 #define DO_HAVE_WAITPID
     /* Use "union wait" instead of int? */
     /* =()<#define @<USE_UNION_WAIT>@_USE_UNION_WAIT>()= */
-#define DO_USE_UNION_WAIT
+#define DONT_USE_UNION_WAIT
     /* How to fork? */
     /* =()<#define FORK()	@<FORK>@()>()= */
 #define FORK()	vfork()
@@ -123,16 +128,16 @@ typedef char		*CSTRING;
 #define DO_HAVE_UNIX_DOMAIN
     /* How should close-on-exec be done? */
     /* =()<#define CLX_@<CLX_STYLE>@>()= */
-#define CLX_IOCTL
+#define CLX_FCNTL
     /* How should non-blocking I/O be done? */
     /* =()<#define NBIO_@<NBIO_STYLE>@>()= */
 #define NBIO_FCNTL
     /* How should resource-totalling be done? */
     /* =()<#define RES_@<RES_STYLE>@>()= */
-#define RES_RUSAGE
+#define RES_TIMES
     /* How to get number of available descriptors? */
     /* =()<#define FDCOUNT_@<FDCOUNT_STYLE>@>()= */
-#define FDCOUNT_GETDTAB
+#define FDCOUNT_SYSCONF
 
     /* If greater than -1, then use [gs]etrlimit to set that many descriptors. */
     /* If -1, then no [gs]etrlimit calls are done. */
@@ -145,13 +150,18 @@ typedef char		*CSTRING;
     /* =()<#define CTYPE(isXXXXX, c)	(@<CTYPE>@)>()= */
 #define CTYPE(isXXXXX, c)	((isascii((c)) && isXXXXX((c))))
 
-
 /*
 **  6.  MISCELLANEOUS CONFIG DATA
 */
     /* Use mmap() to read the active file, or read it in? */
     /* =()<#define ACT_@<ACT_STYLE>@>()= */
-#define ACT_READ
+#define ACT_MMAP
+    /* Should the routines that use mmap() also do a msync(). */
+    /* =()<#define @<MMAP_SYNC>@_MMAP_SYNC>()= */
+#define DONT_MMAP_SYNC
+    /* Does the msync() function take 3 arguments? DO or DONT */
+    /* =()<#define @<MSYNC_3_ARG>@_MSYNC_3_ARG>()= */
+#define DONT_MSYNC_3_ARG
     /* Use our NNTP-server-open routine, or the one in NNTP? */
     /* INND is nicer, but you must install inn.conf files everywhere; NNTP */
     /* is better if you already have lots of /usr/lib/news/server files. */
@@ -190,10 +200,10 @@ typedef char		*CSTRING;
 #define INND_DBZINCORE	1
     /* Should sub-processes get a nice(2) value? */
     /* =()<#define @<INND_NICE_KIDS>@_INND_NICE_KIDS>()= */
-#define DO_INND_NICE_KIDS
+#define DONT_INND_NICE_KIDS
     /* Value for nice(2) call in innd. */
     /* =()<#define INND_NICE_VALUE	@<INND_NICE_VALUE>@>()= */
-#define INND_NICE_VALUE	4
+#define INND_NICE_VALUE	10
     /* Null-terminated list of unknown commands to not log to syslog. */
     /* =()<#define INND_QUIET_BADLIST	@<INND_QUIET_BADLIST>@>()= */
 #define INND_QUIET_BADLIST	NULL
@@ -233,6 +243,9 @@ typedef char		*CSTRING;
     /* INND closes channel if inactive this long (seconds). */
     /* =()<#define PEER_TIMEOUT	@<PEER_TIMEOUT>@>()= */
 #define PEER_TIMEOUT	(1 * 60 * 60)
+    /* NNRP exits if first command doesn't arrive within this time (seconds).*/
+    /* =()<#define INITIAL_TIMEOUT	@<INIT_CLIENT_TIMEOUT>@>()= */
+#define INITIAL_TIMEOUT	30
     /* NNRP exits if inactive this long (seconds). */
     /* =()<#define CLIENT_TIMEOUT	@<CLIENT_TIMEOUT>@>()= */
 #define CLIENT_TIMEOUT	(2 * 60 * 60)
@@ -244,7 +257,7 @@ typedef char		*CSTRING;
 #define NNRP_LOADLIMIT	16
     /* Don't readdir() spool dir if same group within this many seconds. */
     /* =()<#define NNRP_RESCAN_DELAY	@<NNRP_RESCAN_DELAY>@>()= */
-#define NNRP_RESCAN_DELAY	10
+#define NNRP_RESCAN_DELAY	60
     /* Do gethostbyaddr on client addresses in nnrp? */
     /* =()<#define @<NNRP_GETHOSTBYADDR>@_NNRP_GETHOSTBYADDR>()= */
 #define DO_NNRP_GETHOSTBYADDR
@@ -254,6 +267,12 @@ typedef char		*CSTRING;
     /*  Strip Sender from posts that did authenticate? */
     /* =()<#define @<NNRP_AUTH_SENDER>@_NNRP_AUTH_SENDER>()= */
 #define DONT_NNRP_AUTH_SENDER
+    /*  Make life easy for sucking feeds. DO or DONT */
+    /* =()<#define @<LIKE_PULLERS>@_LIKE_PULLERS>()= */
+#define DONT_LIKE_PULLERS
+    /* Allow the NEWNEWS NNTP command? */
+    /* =()<#define @<ALLOW_NEWNEWS>@_ALLOW_NEWNEWS>()= */
+#define DONT_ALLOW_NEWNEWS
     /* How many read/write failures until channel is put to sleep or closed? */
     /* =()<#define BAD_IO_COUNT	@<BAD_IO_COUNT>@>()= */
 #define BAD_IO_COUNT	5
@@ -283,7 +302,10 @@ typedef char		*CSTRING;
 #define CHANNEL_INACTIVE_TIME	(10 * 60)
     /* Put nntplink info (filename) into the log? */
     /* =()<#define @<NNTPLINK_LOG>@_NNTPLINK_LOG>()= */
-#define DONT_NNTPLINK_LOG
+#define DO_NNTPLINK_LOG
+    /* Put article size into the log? */
+    /* =()<#define @<LOG_SIZE>@_LOG_SIZE>()= */
+#define DO_LOG_SIZE
     /* Log by host IP address, rather than from Path line? */
     /* =()<#define @<IPADDR_LOG>@_IPADDR_LOG>()= */
 #define DONT_IPADDR_LOG
@@ -308,6 +330,13 @@ typedef char		*CSTRING;
     /* Byte limit on locally-posted articles; 0 to disable the check. */
     /* =()<#define LOCAL_MAX_ARTSIZE	@<LOCAL_MAX_ARTSIZE>@>()= */
 #define LOCAL_MAX_ARTSIZE	0
+    /* Default number of bytes to hold in memory when buffered. */
+    /* =()<#define SITE_BUFFER_SIZE	@<SITE_BUFFER_SIZE>@>()= */
+#define SITE_BUFFER_SIZE	(16 * 1024)
+    /* Should Inn be calling setsockopt() on network fds. */
+    /* =()<#define @<SET_SOCKOPT>@_SET_SOCKOPT @<SET_SOCKOPT>@>()= */
+#define DO_SET_SOCKOPT DO
+
 
     /* Function that returns no value, and a pointer to it. */
     /* =()<#define FUNCTYPE	@<FUNCTYPE>@>()= */
@@ -318,11 +347,10 @@ typedef FUNCTYPE	(*FUNCPTR)();
 
     /* While reading input, if we have less than LOW_WATER bytes free, we
      * use the current buffersize as input to GROW_AMOUNT to determine how
-     * much to realloc.  (Doubling seems to be the best thing right now.)
-     * Growth must be at least NNTP_STRLEN bytes! */
+     * much to realloc.  Growth must be at least NNTP_STRLEN bytes! */
 #define START_BUFF_SIZE		(4 * 1024)
 #define LOW_WATER		(1 * 1024)
-#define GROW_AMOUNT(x)		((x))
+#define GROW_AMOUNT(x)		((x) < 128 * 1024 ? (x) : 128 * 1024)
 
     /* Some debuggers might need this set to an empty string. */
 #define STATIC			static
@@ -339,9 +367,10 @@ typedef int		BOOL;
 #define NNTP_PORT		119
 #define TRUE			1
 #define FALSE			0
-#define MAXLISTEN		5
+#define MAXLISTEN		25
 #define STDIN			0
 #define STDOUT			1
+#define STDERR			2
 #define PIPE_READ		0
 #define PIPE_WRITE		1
 #define DATE_FUZZ		(24L * 60L * 60L)
@@ -365,6 +394,7 @@ typedef int		BOOL;
 #define FEED_REPLIC		'R'
 #define FEED_SITE		's'
 #define FEED_TIMERECEIVED	't'
+#define FEED_TIMEPOSTED		'p'
 #define HIS_BADCHAR		'_'
 #define HIS_FIELDSEP		'\t'
 #define HIS_NOEXP		"-"
@@ -385,4 +415,23 @@ typedef int		BOOL;
 #define SIG_SEPARATOR		"-- \n"
 #define SUB_DEFAULT		FALSE
 #define SUB_NEGATE		'!'
+#define SUB_POISON		'@'
 #define LOOPBACK_HOST		"127.0.0.1"
+
+/*
+**  13.  TCL Support
+*/
+
+    /* =()<#define @<TCL_SUPPORT>@_TCL>()= */
+#define DONT_TCL
+
+
+/*
+**  17.  Perl Support
+*/
+
+    /* =()<#define @<PERL_SUPPORT>@_PERL>()= */
+#define DONT_PERL
+
+
+#endif /* ! defined (__configdata_h__) */
