@@ -1,4 +1,4 @@
-/*  $Revision: 1.13 $
+/*  $Revision: 1.15 $
 **
 **  Here be declarations of functions in the InterNetNews library.
 */
@@ -6,7 +6,7 @@
 /* Memory allocation. */
     /* Worst-case alignment, in order to shut lint up. */
     /* =()<typedef @<ALIGNPTR>@	*ALIGNPTR;>()= */
-typedef int	*ALIGNPTR;
+typedef long	*ALIGNPTR;
 extern ALIGNPTR	xmalloc();
 extern ALIGNPTR	xrealloc();
 
@@ -47,8 +47,9 @@ extern time_t	parsedate();
 extern int	GetTimeInfo();
 
 /* Miscellaneous. */
+extern int	getfdcount();
 extern int	wildmat();
-extern int	waitnb();
+extern PID_T	waitnb();
 extern int	xread();
 extern int	xwrite();
 extern int	xwritev();
@@ -60,4 +61,5 @@ extern void	Radix32();
 extern char	*INNVersion();
 extern char	*ReadInDescriptor();
 extern char	*ReadInFile();
+extern void	TempName();
 extern FILE	*xfopena();
