@@ -1,5 +1,5 @@
 /*
- * $Revision: 1.21 $
+ * $Revision: 1.22 $
  */
 
 
@@ -13,8 +13,8 @@
 **  Try to avoid changing these.
 */
 #define RELEASE "1"
-#define PATCHLEVEL "5"
-#define DATE "28-Nov-1996"
+#define PATCHLEVEL "5.1"
+#define DATE "17-Dec-1996"
 
 
 /*
