@@ -1,4 +1,4 @@
-/*  $Revision: 1.22 $
+/*  $Revision: 1.33 $
 **
 **  Here be #define's for filenames, socket names, environment variables,
 **  and so on.  The numbers refer to sections in the config.dist file.
@@ -9,36 +9,40 @@
 **  7.  PATHS TO COMMON PROGRAMS
 */
     /* =()<#define _PATH_INND	"@<_PATH_INND>@">()= */
-#define _PATH_INND	"/usr/lib/news/etc/innd"
+#define _PATH_INND	"/var/spool/news/etc/innd"
     /* =()<#define _PATH_INNDSTART	"@<_PATH_INNDSTART>@">()= */
-#define _PATH_INNDSTART	"/usr/lib/news/etc/inndstart"
+#define _PATH_INNDSTART	"/var/spool/news/etc/inndstart"
     /* =()<#define _PATH_SENDMAIL	"@<_PATH_SENDMAIL>@">()= */
-#define _PATH_SENDMAIL	"/usr/sbin/sendmail -t"
+#define _PATH_SENDMAIL	"/usr/lib/sendmail -t"
     /* =()<#define _PATH_SH		"@<_PATH_SH>@">()= */
-#define _PATH_SH		"/usr/contrib/bin/bash"
+#define _PATH_SH		"/bin/sh"
     /* =()<#define _PATH_NNRPD		"@<_PATH_NNRPD>@">()= */
-#define _PATH_NNRPD		"/usr/lib/news/etc/in.nnrpd"
+#define _PATH_NNRPD		"/var/spool/news/etc/in.nnrpd"
     /* =()<#define _PATH_NNTPD          "@<_PATH_NNTPD>@">()= */
-#define _PATH_NNTPD          "/usr/lib/news/etc/in.nnrpd"
+#define _PATH_NNTPD          "/var/spool/news/etc/in.nnrpd"
     /* =()<#define _PATH_NNQRD		"@<_PATH_NNQRD>@">()= */
-#define _PATH_NNQRD		"/usr/lib/news/etc/in.nnrpd"
+#define _PATH_NNQRD		"/var/spool/news/etc/in.nnrpd"
     /* =()<#define _PATH_COMPRESS	"@<_PATH_COMPRESS>@">()= */
-#define _PATH_COMPRESS	"/usr/bin/compress"
+#define _PATH_COMPRESS	"/bin/compress"
     /* =()<#define _PATH_RNEWS	"@<_PATH_RNEWS>@">()= */
 #define _PATH_RNEWS	"/usr/local/bin/rnews"
+    /* =()<#define _PATH_AUTHDIR	"@<_PATH_AUTHDIR>@">()= */
+#define _PATH_AUTHDIR	"/var/spool/news/auth.dir"
     /* =()<#define _PATH_NEWSBIN	"@<_PATH_NEWSBIN>@">()= */
-#define _PATH_NEWSBIN	"/usr/lib/news/bin"
+#define _PATH_NEWSBIN	"/var/spool/news/bin"
     /* =()<#define _PATH_TMP	"@<_PATH_TMP>@">()= */
 #define _PATH_TMP	"/tmp"
+    /* =()<#define _PATH_GZIP	"@<_PATH_GZIP>@">()= */
+#define _PATH_GZIP	"/usr/contrib/bin/gzip"
 
 
 /*
 **  8.  PATHS RELATED TO THE SPOOL DIRECTORY
 */
     /* =()<#define _PATH_SPOOL		"@<_PATH_SPOOL>@">()= */
-#define _PATH_SPOOL		"/var/spool/news"
+#define _PATH_SPOOL		"/var/spool/news/articles"
     /* =()<#define _PATH_OVERVIEWDIR		"@<_PATH_OVERVIEWDIR>@">()= */
-#define _PATH_OVERVIEWDIR		"/var/spool/news"
+#define _PATH_OVERVIEWDIR		"/var/spool/news/overview"
     /* =()<#define _PATH_OVERVIEW		"@<_PATH_OVERVIEW>@">()= */
 #define _PATH_OVERVIEW		".overview"
     /* =()<#define _PATH_SPOOLNEWS	"@<_PATH_SPOOLNEWS>@">()= */
@@ -57,9 +61,9 @@
     /* =()<#define _PATH_RNEWS_DUP_LOG	"@<_PATH_RNEWS_DUP_LOG>@">()= */
 #define _PATH_RNEWS_DUP_LOG	"/dev/null"
     /* =()<#define _PATH_RNEWSPROGS	"@<_PATH_RNEWSPROGS>@">()= */
-#define _PATH_RNEWSPROGS	"/usr/lib/news/bin/rnews"
+#define _PATH_RNEWSPROGS	"/var/spool/news/bin/rnews"
     /* =()<#define _PATH_CONTROLPROGS	"@<_PATH_CONTROLPROGS>@">()= */
-#define _PATH_CONTROLPROGS	"/usr/lib/news/bin/control"
+#define _PATH_CONTROLPROGS	"/var/spool/news/bin/control"
     /* =()<#define _PATH_BADCONTROLPROG	"@<_PATH_BADCONTROLPROG>@">()= */
 #define _PATH_BADCONTROLPROG	"default"
 
@@ -68,68 +72,75 @@
 **  10.  SOCKETS CREATED BY INND OR CLIENTS
 */
     /* =()<#define _PATH_INNDDIR	"@<_PATH_INNDDIR>@">()= */
-#define _PATH_INNDDIR	"/usr/lib/news/innd"
+#define _PATH_INNDDIR	"/var/spool/news/innd"
     /* =()<#define _PATH_NNTPCONNECT	"@<_PATH_NNTPCONNECT>@">()= */
-#define _PATH_NNTPCONNECT	"/usr/lib/news/innd/nntpin"
+#define _PATH_NNTPCONNECT	"/var/spool/news/innd/nntpin"
     /* =()<#define _PATH_NEWSCONTROL	"@<_PATH_NEWSCONTROL>@">()= */
-#define _PATH_NEWSCONTROL	"/usr/lib/news/innd/control"
+#define _PATH_NEWSCONTROL	"/var/spool/news/innd/control"
     /* =()<#define _PATH_TEMPSOCK	"@<_PATH_TEMPSOCK>@">()= */
-#define _PATH_TEMPSOCK	"/usr/lib/news/innd/ctlinndXXXXXX"
+#define _PATH_TEMPSOCK	"/var/spool/news/innd/ctlinndXXXXXX"
 
 
 /*
 **  11.  LOG AND CONFIG FILES
 */
     /* =()<#define _PATH_NEWSLIB	"@<_PATH_NEWSLIB>@">()= */
-#define _PATH_NEWSLIB	"/usr/lib/news"
+#define _PATH_NEWSLIB	"/var/spool/news/lib"
     /* =()<#define _PATH_LOGFILE		"@<_PATH_LOGFILE>@">()= */
 #define _PATH_LOGFILE		"/var/log/news/news"
     /* =()<#define _PATH_ERRLOG	"@<_PATH_ERRLOG>@">()= */
 #define _PATH_ERRLOG	"/var/log/news/errlog"
     /* =()<#define _PATH_SERVERPID	"@<_PATH_SERVERPID>@">()= */
-#define _PATH_SERVERPID	"/usr/lib/news/innd/innd.pid"
+#define _PATH_SERVERPID	"/var/spool/news/innd/innd.pid"
     /* =()<#define _PATH_NEWSFEEDS	"@<_PATH_NEWSFEEDS>@">()= */
-#define _PATH_NEWSFEEDS	"/usr/lib/news/newsfeeds"
+#define _PATH_NEWSFEEDS	"/var/spool/news/lib/newsfeeds"
     /* =()<#define _PATH_HISTORY	"@<_PATH_HISTORY>@">()= */
-#define _PATH_HISTORY	"/usr/lib/news/history"
+#define _PATH_HISTORY	"/var/spool/news/history"
     /* =()<#define _PATH_INNDHOSTS	"@<_PATH_INNDHOSTS>@">()= */
-#define _PATH_INNDHOSTS	"/usr/lib/news/hosts.nntp"
+#define _PATH_INNDHOSTS	"/var/spool/news/lib/hosts.nntp"
     /* =()<#define _PATH_ACTIVE	"@<_PATH_ACTIVE>@">()= */
-#define _PATH_ACTIVE	"/usr/lib/news/active"
+#define _PATH_ACTIVE	"/var/spool/news/active"
     /* =()<#define _PATH_NEWACTIVE	"@<_PATH_NEWACTIVE>@">()= */
-#define _PATH_NEWACTIVE	"/usr/lib/news/active.tmp"
+#define _PATH_NEWACTIVE	"/var/spool/news/active.tmp"
     /* =()<#define _PATH_OLDACTIVE	"@<_PATH_OLDACTIVE>@">()= */
-#define _PATH_OLDACTIVE	"/usr/lib/news/active.old"
+#define _PATH_OLDACTIVE	"/var/spool/news/active.old"
     /* =()<#define _PATH_ACTIVETIMES	"@<_PATH_ACTIVETIMES>@">()= */
-#define _PATH_ACTIVETIMES	"/usr/lib/news/active.times"
+#define _PATH_ACTIVETIMES	"/var/spool/news/active.times"
     /* =()<#define _PATH_BATCHDIR	"@<_PATH_BATCHDIR>@">()= */
 #define _PATH_BATCHDIR	"/var/spool/news/out.going"
     /* =()<#define _PATH_ARCHIVEDIR	"@<_PATH_ARCHIVEDIR>@">()= */
 #define _PATH_ARCHIVEDIR	"/var/spool/news/news.archive"
     /* =()<#define _PATH_DISTPATS	"@<_PATH_DISTPATS>@">()= */
-#define _PATH_DISTPATS	"/usr/lib/news/distrib.pats"
+#define _PATH_DISTPATS	"/var/spool/news/lib/distrib.pats"
     /* =()<#define _PATH_NNRPDIST	"@<_PATH_NNRPDIST>@">()= */
-#define _PATH_NNRPDIST	"/usr/lib/news/distributions"
+#define _PATH_NNRPDIST	"/var/spool/news/lib/distributions"
+    /* =()<#define _PATH_NNRPSUBS       "@<_PATH_NNRPSUBS>@">()= */
+#define _PATH_NNRPSUBS       "/var/spool/news/lib/subscriptions"
     /* =()<#define _PATH_NEWSGROUPS	"@<_PATH_NEWSGROUPS>@">()= */
-#define _PATH_NEWSGROUPS	"/usr/lib/news/newsgroups"
+#define _PATH_NEWSGROUPS	"/var/spool/news/lib/newsgroups"
     /* =()<#define _PATH_CONFIG	"@<_PATH_CONFIG>@">()= */
-#define _PATH_CONFIG	"/usr/lib/news/inn.conf"
+#define _PATH_CONFIG	"/var/spool/news/lib/inn.conf"
     /* =()<#define _PATH_CLIENTACTIVE	"@<_PATH_CLIENTACTIVE>@">()= */
-#define _PATH_CLIENTACTIVE	"/usr/lib/news/active"
+#define _PATH_CLIENTACTIVE	"/var/spool/news/active"
     /* =()<#define _PATH_TEMPACTIVE	"@<_PATH_TEMPACTIVE>@">()= */
 #define _PATH_TEMPACTIVE	"/tmp/activeXXXXXX"
+    /* =()<#define _PATH_TEMPMODERATORS	"@<_PATH_TEMPMODERATORS>@">()= */
+#define _PATH_TEMPMODERATORS	"/var/tmp/moderatorsXXXXXX"
     /* =()<#define _PATH_MODERATORS	"@<_PATH_MODERATORS>@">()= */
-#define _PATH_MODERATORS	"/usr/lib/news/moderators"
+#define _PATH_MODERATORS	"/var/spool/news/lib/moderators"
     /* =()<#define _PATH_SERVER	"@<_PATH_SERVER>@">()= */
-#define _PATH_SERVER	"/usr/lib/news/server"
+#define _PATH_SERVER	"/var/spool/news/lib/server"
     /* =()<#define _PATH_NNTPPASS	"@<_PATH_NNTPPASS>@">()= */
-#define _PATH_NNTPPASS	"/usr/lib/news/passwd.nntp"
+#define _PATH_NNTPPASS	"/var/spool/news/lib/passwd.nntp"
     /* =()<#define _PATH_NNRPACCESS	"@<_PATH_NNRPACCESS>@">()= */
-#define _PATH_NNRPACCESS	"/usr/lib/news/nnrp.access"
+#define _PATH_NNRPACCESS	"/var/spool/news/lib/nnrp.access"
     /* =()<#define _PATH_EXPIRECTL	"@<_PATH_EXPIRECTL>@">()= */
-#define _PATH_EXPIRECTL	"/usr/lib/news/expire.ctl"
+#define _PATH_EXPIRECTL	"/var/spool/news/lib/expire.ctl"
     /* =()<#define _PATH_SCHEMA	"@<_PATH_SCHEMA>@">()= */
-#define _PATH_SCHEMA	"/usr/lib/news/overview.fmt"
+#define _PATH_SCHEMA	"/var/spool/news/lib/overview.fmt"
+    /* =()<#define _PATH_XBATCHES	"@<_PATH_XBATCHES>@">()= */
+#define _PATH_XBATCHES	"/var/news/spool/in.coming"
+
 
 
 /*
@@ -166,3 +177,36 @@
 #define _CONF_CONTENTTYPE	"mime-contenttype"
     /* Default encoding */
 #define _CONF_ENCODING		"mime-encoding"
+
+
+/*
+**  13.  TCL Support
+*/
+
+/* =()<#define _PATH_TCL_STARTUP	"@<_PATH_TCL_STARTUP>@">()= */
+#define _PATH_TCL_STARTUP	"/usr/news/bin/control/startup.tcl"
+
+/* =()<#define _PATH_TCL_FILTER	"@<_PATH_TCL_FILTER>@">()= */
+#define _PATH_TCL_FILTER	"/usr/news/bin/control/filter.tcl"
+
+
+/*
+** 15. Local Configuration
+*/
+
+/*  =()<#define _PATH_NEWSHOME	"@<_PATH_NEWSHOME>@">()=  */
+#define _PATH_NEWSHOME	"/usr/news"
+
+
+/*
+**  17.  PERL Support
+*/
+
+/* =()<#define _PATH_PERL_STARTUP_INND	"@<_PATH_PERL_STARTUP_INND>@">()= */
+#define _PATH_PERL_STARTUP_INND	"/usr/news/bin/control/startup_innd.pl"
+
+/* =()<#define _PATH_PERL_FILTER_INND	"@<_PATH_PERL_FILTER_INND>@">()= */
+#define _PATH_PERL_FILTER_INND	"/usr/news/bin/control/filter_innd.pl"
+
+/* =()<#define _PATH_PERL_FILTER_NNRPD	"@<_PATH_PERL_FILTER_NNRPD>@">()= */
+#define _PATH_PERL_FILTER_NNRPD	"/usr/news/bin/control/filter_nnrpd.pl"
