@@ -1,4 +1,4 @@
-/*  $Revision: 1.67 $
+/*  $Revision: 1.68 $
 **
 **  Article-processing.
 */
@@ -140,6 +140,8 @@ ARTHEADER	ARTheaders[] = {
 
 ARTHEADER	*ARTheadersENDOF = ENDOF(ARTheaders);
 
+const char *pathForPerl ;
+
 
 /*
 **
@@ -1791,6 +1793,7 @@ ARTpost(cp, Replic, ihave)
     }
 
 #if defined(DO_PERL)
+    pathForPerl = HeaderFind(article->Data, "Path", 4) ;
     if ((perlrc = (char *)HandleArticle()) != NULL) {
         (void)sprintf(buff, "%d %s", NNTP_REJECTIT_VAL, perlrc);
         syslog(L_NOTICE, "rejecting[perl] %s %s", HDR(_message_id), buff);
