@@ -1,4 +1,4 @@
-/*  $Revision: 1.35 $
+/*  $Revision: 1.53 $
 **
 **  Routines for the control channel.  Create a Unix-domain datagram socket
 **  that processes on the local server send messages to.  The control
@@ -6,10 +6,18 @@
 **  special functions.  We use datagrams so that we don't need to do an
 **  accept() and tie up another descriptor.  Recvfrom seems to be broken on
 **  several systems, so the client passes in the socket name.
+**
+**  This module completely rips away all pretense of software layering.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 #include "inndcomm.h"
+#if	defined(DO_HAVE_UNIX_DOMAIN)
 #include <sys/un.h>
+#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
 
 
 /*
@@ -20,19 +28,21 @@ typedef struct _CCDISPATCH {
     char	Name;
     int		argc;
     STRING	(*Function)();
-    BOOL	FreeResult;
 } CCDISPATCH;
 
 
 STATIC STRING	CCaddhist();
+STATIC STRING	CCallow();
 STATIC STRING	CCbegin();
 STATIC STRING	CCchgroup();
 STATIC STRING	CCdrop();
+STATIC STRING	CCfeedinfo();
 STATIC STRING	CCflush();
 STATIC STRING	CCflushlogs();
 STATIC STRING	CCgo();
 STATIC STRING	CChangup();
 STATIC STRING	CCreserve();
+STATIC STRING	CClogmode();
 STATIC STRING	CCmode();
 STATIC STRING	CCname();
 STATIC STRING	CCnewgroup();
@@ -40,6 +50,7 @@ STATIC STRING	CCparam();
 STATIC STRING	CCpause();
 STATIC STRING	CCreaders();
 STATIC STRING	CCrefile();
+STATIC STRING	CCreject();
 STATIC STRING	CCreload();
 STATIC STRING	CCrenumber();
 STATIC STRING	CCrmgroup();
@@ -50,47 +61,67 @@ STATIC STRING	CCthrottle();
 STATIC STRING	CCtrace();
 STATIC STRING	CCxabort();
 STATIC STRING	CCxexec();
+#if defined(DO_TCL)
+STATIC STRING	CCfilter();
+#endif /* defined(DO_TCL) */
+#if defined(DO_PERL)
+STATIC STRING	CCperl();
+#endif /* defined(DO_PERL) */
 
 STATIC char		CCpath[] = _PATH_NEWSCONTROL;
-STATIC char		CCerrbuff[80];
 STATIC char		**CCargv;
 STATIC char		CCnosite[] = "1 No such site";
 STATIC char		CCwrongtype[] = "1 Wrong site type";
 STATIC char		CCnogroup[] = "1 No such group";
 STATIC char		CCnochannel[] = "1 No such channel";
+STATIC char		CCnoreason[] = "1 Empty reason";
+STATIC char		CCbigreason[] = "1 Reason too long";
 STATIC char		CCnotrunning[] = "1 Must be running";
+STATIC BUFFER		CCreply;
 STATIC CHANNEL		*CCchan;
+STATIC int		CCwriter;
 STATIC CCDISPATCH	CCcommands[] = {
-    {	SC_ADDHIST,	4, CCaddhist,	FALSE	},
-    {	SC_BEGIN,	1, CCbegin,	FALSE	},
-    {	SC_CANCEL,	1, CCcancel,	FALSE	},
-    {	SC_CHANGEGROUP,	2, CCchgroup,	FALSE	},
-    {	SC_CHECKFILE,	0, CCcheckfile,	FALSE	},
-    {	SC_DROP,	1, CCdrop,	FALSE	},
-    {	SC_FLUSH,	1, CCflush,	FALSE	},
-    {	SC_FLUSHLOGS,	0, CCflushlogs,	FALSE	},
-    {	SC_GO,		1, CCgo,	FALSE	},
-    {	SC_HANGUP,	1, CChangup,	FALSE	},
-    {	SC_MODE,	0, CCmode,	TRUE	},
-    {	SC_NAME,	1, CCname,	TRUE	},
-    {	SC_NEWGROUP,	3, CCnewgroup,	FALSE	},
-    {	SC_PARAM,	2, CCparam,	FALSE	},
-    {	SC_PAUSE,	1, CCpause,	FALSE	},
-    {	SC_READERS,	2, CCreaders,	FALSE	},
-    {	SC_REFILE,	2, CCrefile,	FALSE	},
-    {	SC_RENUMBER,	1, CCrenumber,	FALSE	},
-    {	SC_RELOAD,	2, CCreload,	FALSE	},
-    {	SC_RESERVE,	1, CCreserve,	FALSE	},
-    {	SC_RMGROUP,	1, CCrmgroup,	FALSE	},
-    {	SC_SEND,	2, CCsend,	FALSE	},
-    {	SC_SHUTDOWN,	1, CCshutdown,	FALSE	},
-    {	SC_SIGNAL,	2, CCsignal,	FALSE	},
-    {	SC_THROTTLE,	1, CCthrottle,	FALSE	},
-    {	SC_TRACE,	2, CCtrace,	FALSE	},
-    {	SC_XABORT,	1, CCxabort,	FALSE	},
-    {	SC_XEXEC,	1, CCxexec,	FALSE	}
+    {	SC_ADDHIST,	5, CCaddhist	},
+    {	SC_ALLOW,	1, CCallow	},
+    {	SC_BEGIN,	1, CCbegin	},
+    {	SC_CANCEL,	1, CCcancel	},
+    {	SC_CHANGEGROUP,	2, CCchgroup	},
+    {	SC_CHECKFILE,	0, CCcheckfile	},
+    {	SC_DROP,	1, CCdrop	},
+    {	SC_FEEDINFO,	1, CCfeedinfo	},
+#if defined(DO_TCL)
+    {	SC_FILTER,	1, CCfilter     },
+#endif /* defined(DO_TCL) */
+#if defined(DO_PERL)
+    {	SC_PERL,	1, CCperl       },
+#endif /* defined(DO_PERL) */
+    {	SC_FLUSH,	1, CCflush	},
+    {	SC_FLUSHLOGS,	0, CCflushlogs	},
+    {	SC_GO,		1, CCgo		},
+    {	SC_HANGUP,	1, CChangup	},
+    {	SC_LOGMODE,	0, CClogmode	},
+    {	SC_MODE,	0, CCmode	},
+    {	SC_NAME,	1, CCname	},
+    {	SC_NEWGROUP,	3, CCnewgroup	},
+    {	SC_PARAM,	2, CCparam	},
+    {	SC_PAUSE,	1, CCpause	},
+    {	SC_READERS,	2, CCreaders	},
+    {	SC_REFILE,	2, CCrefile	},
+    {	SC_REJECT,	1, CCreject	},
+    {	SC_RENUMBER,	1, CCrenumber	},
+    {	SC_RELOAD,	2, CCreload	},
+    {	SC_RESERVE,	1, CCreserve	},
+    {	SC_RMGROUP,	1, CCrmgroup	},
+    {	SC_SEND,	2, CCsend	},
+    {	SC_SHUTDOWN,	1, CCshutdown	},
+    {	SC_SIGNAL,	2, CCsignal	},
+    {	SC_THROTTLE,	1, CCthrottle	},
+    {	SC_TRACE,	2, CCtrace	},
+    {	SC_XABORT,	1, CCxabort	},
+    {	SC_XEXEC,	1, CCxexec	}
 };
 
+STATIC SIGHANDLER	CCresetup();
 
 
 void
@@ -111,6 +142,29 @@ CCcopyargv(av)
 }
 
 
+/*
+**  Return a string representing our operating mode.
+*/
+STATIC STRING
+CCcurrmode()
+{
+    static char		buff[32];
+
+    /* Server's mode. */
+    switch (Mode) {
+    default:
+	(void)sprintf(buff, "Unknown %d", Mode);
+	return buff;
+    case OMrunning:
+	return "running";
+    case OMpaused:
+	return "paused";
+    case OMthrottled:
+	return "throttled";
+    }
+}
+
+
 /*
 **  Add <> around Message-ID if needed.
 */
@@ -173,18 +227,52 @@ CCaddhist(av)
     static char		DIGITS[] = "0123456789";
     ARTDATA		Data;
     STRING		p;
+    BOOL		ok;
 
-    /* Check the fields fields. */
+    /* Check the fields. */
+    if ((p = CCgetid(av[0], &Data.MessageID)) != NULL)
+	return p;
+    if (HIShavearticle(Data.MessageID))
+	return "1 Duplicate";
     if (strspn(av[1], DIGITS) != strlen(av[1]))
-	return "1 Bad posted date";
-    Data.Posted = atol(av[1]);
+	return "1 Bad arrival date";
+    Data.Arrived = atol(av[1]);
     if (strspn(av[2], DIGITS) != strlen(av[2]))
 	return "1 Bad expiration date";
     Data.Expires = atol(av[2]);
-    if ((p = CCgetid(av[0], &Data.MessageID)) != NULL)
-	return p;
+    if (strspn(av[3], DIGITS) != strlen(av[3]))
+	return "1 Bad posted date";
+    Data.Posted = atol(av[3]);
 
-    return !HISwrite(&Data, av[3]) ? "1 Write failed" : NULL;
+    if (Mode == OMrunning)
+	ok = HISwrite(&Data, av[4]);
+    else {
+	/* Possible race condition, but documented in ctlinnd manpage. */
+	HISsetup();
+	ok = HISwrite(&Data, av[4]);
+	HISclose();
+    }
+    return ok ? NULL : "1 Write failed";
+}
+
+
+/*
+**  Do the work to allow foreign connectiosn.
+*/
+STATIC STRING
+CCallow(av)
+    char	*av[];
+{
+    char	*p;
+
+    if (RejectReason == NULL)
+	return "1 Already allowed";
+    p = av[0];
+    if (*p && !EQ(p, RejectReason))
+	return "1 Wrong reason";
+    DISPOSE(RejectReason);
+    RejectReason = NULL;
+    return NULL;
 }
 
 
@@ -203,6 +291,7 @@ CCbegin(av)
     register NEWSGROUP	*ngp;
     STRING		error;
     char		*subbed;
+    char		*poison;
 
     /* If site already exists, drop it. */
     if (SITEfind(av[0]) != NULL && (p = CCdrop(av)) != NULL)
@@ -231,16 +320,20 @@ CCbegin(av)
 	    nSites++;
 	    RENEW(Sites, SITE, nSites);
 	    sp = &Sites[nSites - 1];
-	    for (i = nGroups, ngp = Groups; --i >= 0; ngp++)
+	    sp->Next = sp->Prev = NOSITE;
+	    for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
 		RENEW(ngp->Sites, int, nSites);
+		RENEW(ngp->Poison, int, nSites);
+	    }
 	}
-	SITElinkall();
     }
 
     /* Parse. */
     subbed = NEW(char, nGroups);
-    error = SITEparseone(p, sp, subbed);
+    poison = NEW(char, nGroups);
+    error = SITEparseone(p, sp, subbed, poison);
     DISPOSE(subbed);
+    DISPOSE(poison);
     if (error != NULL) {
 	DISPOSE(p);
 	syslog(L_ERROR, "%s bad_newsfeeds %s", av[0], error);
@@ -249,6 +342,7 @@ CCbegin(av)
 
     if (sp != &ME && (!SITEsetup(sp) || !SITEfunnelpatch()))
 	return "1 Startup error";
+    SITEforward(sp, "begin");
     return NULL;
 }
 
@@ -264,14 +358,14 @@ CCdochange(ngp, Rest)
     int			length;
     char		*p;
 
-    if (Mode != OMrunning)
-	return CCnotrunning;
-
     if (ngp->Rest[0] == Rest[0]) {
 	length = strlen(Rest);
 	if (ngp->Rest[length] == '\n' && EQn(ngp->Rest, Rest, length))
 	    return "0 Group status unchanged";
     }
+    if (Mode != OMrunning)
+	return CCnotrunning;
+
     p = COPY(ngp->Name);
     if (!ICDchangegroup(ngp, Rest)) {
 	syslog(L_NOTICE, "%s cant change_group %s to %s", LogName, p, Rest);
@@ -353,20 +447,28 @@ CCcheckfile(av)
 
     /* Parse all site entries. */
     strings = SITEreadfile(FALSE);
+    fake.Buffer.Size = 0;
+    fake.Buffer.Data = NULL;
     for (errors = 0, i = 0; (p = strings[i]) != NULL; i++) {
-	if ((error = SITEparseone(p, &fake, (char *)NULL)) != NULL) {
-	    syslog(L_ERROR, "%.20s bad_newsfeeds %s", p, error);
+	if ((error = SITEparseone(p, &fake, (char *)NULL, (char *)NULL)) != NULL) {
+	    syslog(L_ERROR, "%s bad_newsfeeds %s", MaxLength(p, p), error);
 	    errors++;
 	}
-	DISPOSE(p);
+	SITEfree(&fake);
     }
     DISPOSE(strings);
 
     if (errors == 0)
 	return NULL;
 
-    (void)sprintf(CCerrbuff, "1 Found %d errors -- see syslog", errors);
-    return CCerrbuff;
+    if (CCreply.Size == 0) {
+	/* If we got the "-s" flag, then CCsetup hasn't been called yet. */
+	CCreply.Size = SMBUF;
+	CCreply.Data = NEW(char, CCreply.Size);
+    }
+
+    (void)sprintf(CCreply.Data, "1 Found %d errors -- see syslog", errors);
+    return CCreply.Data;
 }
 
 
@@ -390,14 +492,110 @@ CCdrop(av)
     SITEdrop(sp);
 
     /* Loop over all groups, and if the site is in a group, clobber it. */
-    for (idx = sp - Sites, i = nGroups, ngp = Groups; --i >= 0; ngp++)
+    for (idx = sp - Sites, i = nGroups, ngp = Groups; --i >= 0; ngp++) {
 	for (j = ngp->nSites, ip = ngp->Sites; --j >= 0; ip++)
 	    if (*ip == idx)
 		*ip = NOSITE;
+	for (j = ngp->nPoison, ip = ngp->Poison; --j >= 0; ip++)
+	    if (*ip == idx)
+		*ip = NOSITE;
+    }
 
 	return NULL;
 }
 
+/*
+**  Return info on the feeds for one, or all, sites
+*/
+STATIC STRING
+CCfeedinfo(av)
+    char		*av[];
+{
+    register SITE	*sp;
+    register char	*p;
+    register int	i;
+
+    BUFFset(&CCreply, "0 ", 2);
+    p = av[0];
+    if (*p != '\0') {
+	if ((sp = SITEfind(p)) == NULL)
+	    return "1 No such site";
+
+	SITEinfo(&CCreply, sp, TRUE);
+	while ((sp = SITEfindnext(p, sp)) != NULL)
+	    SITEinfo(&CCreply, sp, TRUE);
+    }
+    else
+	for (i = nSites, sp = Sites; --i >= 0; sp++)
+	    if (sp->Name)
+		SITEinfo(&CCreply, sp, FALSE);
+
+    BUFFappend(&CCreply, "", 1);
+    return CCreply.Data;
+}
+
+
+#if defined(DO_TCL)
+STATIC STRING
+CCfilter(av)
+    char	*av[];
+{
+    char	*p;
+
+    switch (av[0][0]) {
+    default:
+	return "1 Bad flag";
+    case 'y':
+	if (TCLFilterActive)
+	    return "1 tcl filter already enabled";
+	TCLfilter(TRUE);
+	break;
+    case 'n':
+	if (!TCLFilterActive)
+	    return "1 tcl filter already disabled";
+	TCLfilter(FALSE);
+	break;
+    }
+    return NULL;
+}
+#endif /* defined(DO_TCL) */
+
+
+#if defined(DO_PERL)
+
+#include <EXTERN.h>
+#include <perl.h>
+#include <XSUB.h>
+
+extern CV *perl_filter_cv ;
+
+STATIC STRING
+CCperl(av)
+    char	*av[];
+{
+    char	*p;
+    extern int	PerlFilterActive;
+
+    switch (av[0][0]) {
+    default:
+	return "1 Bad flag";
+    case 'y':
+	if (PerlFilterActive)
+	    return "1 Perl filter already enabled";
+        else if (perl_filter_cv == NULL)
+            return "1 Perl filter not defined" ;
+	PerlFilter(TRUE);
+	break;
+    case 'n':
+	if (!PerlFilterActive)
+	    return "1 Perl filter already disabled";
+	PerlFilter(FALSE);
+	break;
+    }
+    return NULL;
+}
+#endif /* defined(DO_PERL) */
+
 
 /*
 **  Flush all sites or one site.
@@ -439,6 +637,7 @@ CCflushlogs(av)
 	return "1 In debug mode";
 
     ICDwrite();
+    syslog(L_NOTICE, "%s flushlogs %s", LogName, CCcurrmode());
     ReopenLog(Log);
     ReopenLog(Errlog);
     return NULL;
@@ -455,14 +654,29 @@ CCgo(av)
     static char	YES[] = "y";
     char	*p;
 
+    p = av[0];
+    if (Reservation && EQ(p, Reservation)) {
+	DISPOSE(Reservation);
+	Reservation = NULL;
+    }
+    if (RejectReason && EQ(p, RejectReason)) {
+	DISPOSE(RejectReason);
+	RejectReason = NULL;
+    }
+
     if (Mode == OMrunning)
 	return "1 Already running";
-    p = av[0];
     if (*p && !EQ(p, ModeReason))
 	return "1 Wrong reason";
-    Mode = OMrunning;
+
+#if defined(DO_PERL)
+    PerlMode(Mode, OMrunning, p);
+#endif /* defined(DO_PERL) */
+    
     DISPOSE(ModeReason);
     ModeReason = NULL;
+    Mode = OMrunning;
+
     if (NNRPReason && NNRPFollows) {
 	av[0] = YES;
 	av[1] = p;
@@ -474,6 +688,7 @@ CCgo(av)
     syslog(L_NOTICE, "%s running", LogName);
     if (ICDneedsetup)
 	ICDsetup(TRUE);
+    SCHANwakeup((POINTER)&Mode);
     return NULL;
 }
 
@@ -503,12 +718,13 @@ CChangup(av)
 	    p = CHANname(cp);
 	    switch (cp->Type) {
 	    default:
-		(void)sprintf(CCerrbuff, "1 Can't close %s", p);
-		return CCerrbuff;
+		(void)sprintf(CCreply.Data, "1 Can't close %s", p);
+		return CCreply.Data;
 	    case CTexploder:
 	    case CTprocess:
 	    case CTfile:
 	    case CTnntp:
+	    case CTreject:
 		syslog(L_NOTICE, "%s hangup", p);
 		CHANclose(cp, p);
 		return NULL;
@@ -530,6 +746,17 @@ CCmode(av)
     register int	i;
     int			h;
     char		buff[BUFSIZ];
+#if defined(DO_PERL)
+    extern int		PerlFilterActive;
+#endif /* defined(DO_PERL) */
+
+    /* nb: We assume here that BUFSIZ is >= 512, and that none of
+     * ModeReason RejectReason Reservation or NNRPReason is longer than
+     * MAX_REASON_LEN bytes (or actually, the average of their lengths is
+     * <= MAX_REASON_LEN).  If this is not true, the sprintf's/strcpy's
+     * below are likely to overflow buff with somewhat nasty
+     * consequences...
+     */
 
     p = buff;
     p += strlen(strcpy(buff, "0 Server "));
@@ -552,15 +779,23 @@ CCmode(av)
 	p += strlen(strcpy(p, ModeReason));
 	break;
     }
+    *p++ = '\n';
+    if (RejectReason) {
+	p += strlen(strcpy(p, "Rejecting "));
+	p += strlen(strcpy(p, RejectReason));
+    }
+    else
+	p += strlen(strcpy(p, "Allowing remote connections"));
 
     /* Server parameters. */
     for (i = 0, h = 0; CHANiter(&h, CTnntp) != NULL; )
 	i++;
     *p++ = '\n';
-    (void)sprintf(p, "Parameters c %d i %d (%d) l %ld o %d t %ld %s %s",
+    (void)sprintf(p, "Parameters c %ld i %d (%d) l %ld o %d t %ld H %d T %d X %d %s %s",
 	    (long)Cutoff / (24L * 60L * 60L),
 	    MaxIncoming, i,
 	    LargestArticle, MaxOutgoing, (long)TimeOut.tv_sec,
+	    RemoteLimit, RemoteTotal, (int) RemoteTimer,
 	    AmSlave ? "slave" : "normal",
 	    AnyIncoming ? "any" : "specified");
     p += strlen(p);
@@ -587,8 +822,44 @@ CCmode(av)
 	(void)sprintf(p, "disabled %s", NNRPReason);
 	p += strlen(p);
     }
-	
-    return COPY(buff);
+
+#if defined(DO_TCL)
+    *p++ = '\n';
+    p += strlen(strcpy(p, "Tcl filtering "));
+    if (TCLFilterActive)
+	p += strlen(strcpy(p, "enabled"));
+    else
+	p += strlen(strcpy(p, "disabled"));
+#endif /* defined(DO_TCL) */
+#if defined(DO_PERL)
+    *p++ = '\n';
+    p += strlen(strcpy(p, "Perl filtering "));
+    if (PerlFilterActive)
+        p += strlen(strcpy(p, "enabled"));
+    else
+        p += strlen(strcpy(p, "disabled"));
+#endif /* defined(DO_PERL) */
+
+     i = strlen(buff);
+    if (CCreply.Size <= i) {
+	CCreply.Size = i;
+	RENEW(CCreply.Data, char, CCreply.Size + 1);
+    }
+    (void)strcpy(CCreply.Data, buff);
+    return CCreply.Data;
+}
+
+
+/*
+**  Log our operating mode (via syslog).
+*/
+/* ARGSUSED */
+STATIC STRING
+CClogmode(av)
+    char		*av[];
+{
+    syslog(L_NOTICE, "%s servermode %s", LogName, CCcurrmode());
+    return NULL;
 }
 
 
@@ -599,38 +870,31 @@ STATIC STRING
 CCname(av)
     char		*av[];
 {
+    static char		NL[] = "\n";
+    static char		NIL[] = "\0";
     register CHANNEL	*cp;
-    register int	j;
-    register int	length;
     register char	*p;
+    register int	count;
     int			i;
-    char		*name;
-    char		buff[SMBUF];
 
     p = av[0];
     if (*p != '\0') {
 	if ((cp = CHANfromdescriptor(atoi(p))) == NULL)
 	    return COPY(CCnochannel);
-	(void)sprintf(buff, "0 %s", CHANname(cp));
-	return COPY(buff);
+	(void)sprintf(CCreply.Data, "0 %s", CHANname(cp));
+	return CCreply.Data;
     }
-    p = NEW(char, 3);
-    (void)strcpy(p, "0 ");
-    j = 2;
-    for (i = 0; (cp = CHANiter(&i, CTany)) != NULL; ) {
+    BUFFset(&CCreply, "0 ", 2);
+    for (count = 0, i = 0; (cp = CHANiter(&i, CTany)) != NULL; ) {
 	if (cp->Type == CTfree)
 	    continue;
-	name = CHANname(cp);
-	length = strlen(name);
-	RENEW(p, char, j + length + 1 + 1);
-	(void)strcpy(&p[j], name);
-	j += length;
-	p[j++] = '\n';
-	p[j] = '\0';
+	if (++count > 1)
+	    BUFFappend(&CCreply, NL, 1);
+	p = CHANname(cp);
+	BUFFappend(&CCreply, p, strlen(p));
     }
-    if (j > 2)
-	p[--j] = '\0';
-    return p;
+    BUFFappend(&CCreply, NIL, 1);
+    return CCreply.Data;
 }
 
 
@@ -649,8 +913,8 @@ CCnewgroup(av)
     char		*Name;
     char		*Rest;
     STRING		who;
-    int			mask;
     char		buff[SMBUF];
+    int			oerrno;
 
     Name = av[0];
     if (Name[0] == '.' || strspn(Name, "0123456789") == strlen(Name))
@@ -660,7 +924,7 @@ CCnewgroup(av)
 	    if (p[1] == '.' || p[1] == '\0')
 		return "1 Double or trailing period in newsgroup name";
 	}
-	else if (ISWHITE(*p) || *p == ':' || *p == '!')
+	else if (ISWHITE(*p) || *p == ':' || *p == '!' || *p == '/')
 	    return "1 Illegal character in newsgroup name";
 
     Rest = av[1];
@@ -669,15 +933,18 @@ CCnewgroup(av)
 	if (CTYPE(isupper, Rest[0]))
 	    Rest[0] = tolower(Rest[0]);
     }
+    if (strlen(Name) + strlen(Rest) > (SIZE_T)(SMBUF - 24))
+	return "1 Name too long";
+
     if ((ngp = NGfind(Name)) != NULL)
 	return CCdochange(ngp, Rest);
 
     /* Update the log of groups created.  Don't use stdio because SunOS
      * 4.1 has broken libc which can't handle fd's greater than 127. */
-    mask = umask(0);
     if ((fd = open(TIMES, O_WRONLY | O_APPEND | O_CREAT, 0664)) < 0) {
-	IOError(WHEN);
+	oerrno = errno;
 	syslog(L_ERROR, "%s cant fopen %s %m", LogName, TIMES);
+	IOError(WHEN, oerrno);
     }
     else {
 	who = av[2];
@@ -685,33 +952,60 @@ CCnewgroup(av)
 	    who = NEWSMASTER;
 	(void)sprintf(buff, "%s %ld %s\n", Name, Now.time, who);
 	if (xwrite(fd, buff, strlen(buff)) < 0) {
-	    IOError(WHEN);
+	    oerrno = errno;
 	    syslog(L_ERROR, "%s cant write %s %m", LogName, TIMES);
+	    IOError(WHEN, oerrno);
 	}
 	if (close(fd) < 0) {
-	    IOError(WHEN);
+	    oerrno = errno;
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, TIMES);
+	    IOError(WHEN, oerrno);
 	}
 	if (AmRoot)
 	    xchown(TIMES);
     }
-    (void)umask(mask);
 
     /* Update the in-core data. */
     if (!ICDnewgroup(Name, Rest))
 	return "1 Failed";
     syslog(L_NOTICE, "%s newgroup %s as %s", LogName, Name, Rest);
 
-    /* Create the spool directory. */
-    for (p = Name; *p; p++)
-	if (*p == '.')
-	    *p = '/';
-    if (!MakeSpoolDirectory(Name))
-	syslog(L_NOTICE, "%s cant mkdir %s %m", LogName, Name);
+    if (*Rest != NF_FLAG_ALIAS) {
+	/* Create the spool directory. */
+	for (p = Name; *p; p++)
+	    if (*p == '.')
+		*p = '/';
+	if (!MakeSpoolDirectory(Name))
+	    syslog(L_NOTICE, "%s cant mkdir %s %m", LogName, Name);
+    }
     return NULL;
 }
 
 
+/*
+**  Parse and set a boolean flag.
+*/
+STATIC BOOL
+CCparsebool(name, bp, value)
+    char	name;
+    BOOL	*bp;
+    char	value;
+{
+    switch (value) {
+    default:
+	return FALSE;
+    case 'y':
+	*bp = FALSE;
+	break;
+    case 'n':
+	*bp = TRUE;
+	break;
+    }
+    syslog(L_NOTICE, "%s changed -%c %c", LogName, name, value);
+    return TRUE;
+}
+
+
 /*
 **  Change a running parameter.
 */
@@ -719,12 +1013,17 @@ STATIC STRING
 CCparam(av)
     char	*av[];
 {
+    static char	BADVAL[] = "1 Bad value";
     char	*p;
 
     p = av[1];
     switch (av[0][0]) {
     default:
 	return "1 Unknown parameter";
+    case 'a':
+	if (!CCparsebool('a', &AnyIncoming, *p))
+	    return BADVAL;
+	break;
     case 'c':
 	Cutoff = atoi(p) * 24 * 60 * 60;
 	syslog(L_NOTICE, "%s changed -c %d", LogName, Cutoff);
@@ -738,12 +1037,8 @@ CCparam(av)
 	syslog(L_NOTICE, "%s changed -l %ld", LogName, LargestArticle);
 	break;
     case 'n':
-	switch (*p) {
-	default:	return "1 Bad value";
-	case 'y':	NNRPFollows = FALSE;	break;
-	case 'n':	NNRPFollows = TRUE;	break;
-	}
-	syslog(L_NOTICE, "%s changed -n %c", LogName, *p);
+	if (!CCparsebool('n', &NNRPFollows, *p))
+	    return BADVAL;
 	break;
     case 'o':
 	MaxOutgoing = atoi(p);
@@ -753,6 +1048,20 @@ CCparam(av)
 	TimeOut.tv_sec = atol(p);
 	syslog(L_NOTICE, "%s changed -t %ld", LogName, (long)TimeOut.tv_sec);
 	break;
+    case 'H':
+	RemoteLimit = atoi(p);
+	syslog(L_NOTICE, "%s changed -H %d", LogName, RemoteLimit);
+	break;
+    case 'T':
+	RemoteTotal = atoi(p);
+	if (RemoteTotal > REMOTETABLESIZE)
+	    RemoteTotal = REMOTETABLESIZE;
+	syslog(L_NOTICE, "%s changed -T %d", LogName, RemoteTotal);
+	break;
+    case 'X':
+	RemoteTimer = (time_t) atoi(p);
+	syslog(L_NOTICE, "%s changed -X %d", LogName, (int) RemoteTimer);
+	break;
     }
     return NULL;
 }
@@ -769,14 +1078,25 @@ CCblock(NewMode, reason)
     static char		NO[] = "n";
     STRING		av[2];
 
+    if (*reason == '\0')
+	return CCnoreason;
+
+    if (strlen(reason) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+	return CCbigreason;
+
     if (Reservation) {
 	if (!EQ(reason, Reservation)) {
-	    (void)sprintf(CCerrbuff, "1 Reserved \"%s\"", Reservation);
-	    return CCerrbuff;
+	    (void)sprintf(CCreply.Data, "1 Reserved \"%s\"", Reservation);
+	    return CCreply.Data;
 	}
 	DISPOSE(Reservation);
 	Reservation = NULL;
     }
+
+#if defined(DO_PERL)
+    PerlMode(Mode, NewMode, reason);
+#endif /* defined(DO_PERL) */
+
     ICDwrite();
     HISclose();
     Mode = NewMode;
@@ -837,7 +1157,12 @@ CCreaders(av)
     case 'n':
 	if (NNRPReason)
 	    return "1 Already not allowing readers";
-	NNRPReason = COPY(av[1]);
+	p = av[1];
+	if (*p == '\0')
+	    return CCnoreason;
+	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+	    return CCbigreason;
+	NNRPReason = COPY(p);
 	break;
     }
     return NULL;
@@ -870,7 +1195,7 @@ CCxexec(av)
 	return "1 Bad value";
 
     JustCleanup();
-    syslog(L_FATAL, "%s execv %s", LogName, CCargv[0]);
+    syslog(L_NOTICE, "%s execv %s", LogName, CCargv[0]);
     (void)execv(CCargv[0], CCargv);
     syslog(L_FATAL, "%s cant execv %s %m", LogName, CCargv[0]);
     _exit(1);
@@ -899,6 +1224,22 @@ CCrefile(av)
 }
 
 
+/*
+**  Reject remote readers.
+*/
+STATIC STRING
+CCreject(av)
+    char	*av[];
+{
+    if (RejectReason)
+	return "1 Already rejecting";
+    if (strlen(av[0]) > MAX_REASON_LEN)	/* MAX_REASON_LEN is as big as is safe */
+	return CCbigreason;
+    RejectReason = COPY(av[0]);
+    return NULL;
+}
+
+
 /*
 **  Re-read all in-core data.
 */
@@ -906,6 +1247,10 @@ STATIC STRING
 CCreload(av)
     char	*av[];
 {
+    static char	BADSCHEMA[] = "1 Can't read schema";
+#if defined(DO_PERL)
+    static char BADPERLRELOAD[] = "1 Failed to define filter_art" ;
+#endif /* defined(DO_PERL) */
     STRING	p;
 
     p = av[0];
@@ -916,6 +1261,14 @@ CCreload(av)
 	HISsetup();
 	ICDwrite();
 	ICDsetup(TRUE);
+	if (!ARTreadschema())
+	    return BADSCHEMA;
+#if defined(DO_TCL)
+	TCLreadfilter();
+#endif /* defined(DO_TCL) */
+#if defined(DO_PERL)
+        PERLreadfilter (_PATH_PERL_FILTER_INND,"filter_art") ;
+#endif /* defined(DO_PERL) */
 	p = "all";
     }
     else if (EQ(p, "active") || EQ(p, "newsfeeds")) {
@@ -929,6 +1282,21 @@ CCreload(av)
     }
     else if (EQ(p, "hosts.nntp"))
 	RCreadlist();
+    else if (EQ(p, "overview.fmt")) {
+	if (!ARTreadschema())
+	    return BADSCHEMA;
+    }
+#if defined(DO_TCL)
+    else if (EQ(p, "filter.tcl")) {
+	TCLreadfilter();
+    }
+#endif /* defined(DO_TCL) */
+#if defined(DO_PERL)
+    else if (EQ(p, "filter.perl")) {
+        if (!PERLreadfilter (_PATH_PERL_FILTER_INND,"filter_art"))
+            return BADPERLRELOAD ;
+    }
+#endif /* defined(DO_PERL) */
     else
 	return "1 Unknown reload type";
 
@@ -944,6 +1312,7 @@ STATIC STRING
 CCrenumber(av)
     char	*av[];
 {
+    static char	CANTRENUMBER[] = "1 Failed (see syslog)";
     char	*p;
     NEWSGROUP	*ngp;
 
@@ -956,10 +1325,10 @@ CCrenumber(av)
 	if ((ngp = NGfind(p)) == NULL)
 	    return CCnogroup;
 	if (!NGrenumber(ngp))
-	    return "1 Failed (see syslog)";
+	    return CANTRENUMBER;
     }
-    else
-	ICDrenumberactive();
+    else if (!ICDrenumberactive())
+	return CANTRENUMBER;
     return NULL;
 }
 
@@ -980,6 +1349,8 @@ CCreserve(av)
 	/* Trying to make a reservation. */
 	if (Reservation)
 	    return "1 Already reserved";
+	if (strlen(p) > MAX_REASON_LEN) /* MAX_REASON_LEN is as big as is safe */
+	    return CCbigreason;
 	Reservation = COPY(p);
     }
     else {
@@ -1002,8 +1373,6 @@ CCrmgroup(av)
 {
     NEWSGROUP	*ngp;
 
-    if (Mode != OMrunning)
-	return CCnotrunning;
     if ((ngp = NGfind(av[0])) == NULL)
 	return CCnogroup;
 
@@ -1077,16 +1446,17 @@ CCsignal(av)
 	return CCnosite;
     if (sp->Type != FTchannel && sp->Type != FTexploder)
 	return CCwrongtype;
-    if (sp->Spooling || sp->Process  < 0)
+    if (sp->Process < 0)
 	return "1 Site has no process";
 
     /* Do it. */
     if (kill(sp->pid, s) < 0) {
 	oerrno = errno;
-	syslog(L_ERROR, "%s cant kill %d %d site %s, %m", sp->Process, s, p);
-	(void)sprintf(CCerrbuff, "1 Can't signal process %d, %s",
-		sp->Process, strerror(oerrno));
-	return CCerrbuff;
+	syslog(L_ERROR, "%s cant kill %ld %d site %s, %m", LogName, 
+		(long) sp->pid, s, p);
+	(void)sprintf(CCreply.Data, "1 Can't signal process %ld, %s",
+		(long) sp->pid, strerror(oerrno));
+	return CCreply.Data;
     }
 
     return NULL;
@@ -1198,16 +1568,20 @@ CCreader(cp)
     register CCDISPATCH	*dp;
     register STRING	p;
     register char	*q;
+    ICC_MSGLENTYPE	bufflen;
+    ICC_PROTOCOLTYPE	protocol ;
 #if	defined(DO_HAVE_UNIX_DOMAIN)
     struct sockaddr_un	client;
+#else
+    int			written;
 #endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
-    BOOL		FreeResult;
     int			i;
     char		buff[BUFSIZ + 2];
     char		copy[BUFSIZ + 2];
     char		*argv[SC_MAXFIELDS + 2];
     int			argc;
     int			len;
+    char		*tbuff ;
 
     if (cp != CCchan) {
 	syslog(L_ERROR, "%s internal CCreader wrong channel 0x%x not 0x%x",
@@ -1215,18 +1589,78 @@ CCreader(cp)
 	return;
     }
 
-    /* Get the message. */
-    i = RECVorREAD(CCchan->fd, buff, sizeof buff - 1);
+#if defined (DO_HAVE_UNIX_DOMAIN)
+    
+    i = RECVorREAD(CCchan->fd, buff, BUFSIZ) ;
     if (i < 0) {
 	syslog(L_ERROR, "%s cant recv CCreader %m", LogName);
 	return;
-    }
-    if (i == 0) {
-	syslog(L_ERROR, "%s cant recv CCreader empty %m", LogName);
+    } else if (i == 0) {
+	syslog(L_ERROR, "%s cant recv CCreader empty", LogName);
+	return;
+    } else if (i < HEADER_SIZE) {
+	syslog(L_ERROR, "%s cant recv CCreader header-length %m", LogName);
 	return;
     }
+
+    memcpy (&protocol,buff,sizeof (protocol)) ;
+    memcpy (&bufflen,buff + sizeof (protocol),sizeof (bufflen)) ;
+    bufflen = ntohs (bufflen) ;
+    
+    if (i != bufflen) {
+	syslog(L_ERROR, "%s cant recv CCreader short-read %m", LogName);
+	return;
+    }
+
+    i -= HEADER_SIZE ;
+    memmove (buff,buff + HEADER_SIZE,i) ;
     buff[i] = '\0';
 
+    if (protocol != ICC_PROTOCOL_1) {
+        syslog(L_ERROR, "%s CCreader protocol mismatch", LogName) ;
+        return ;
+    }
+
+#else  /* defined (DO_HAVE_UNIX_DOMAIN) */
+
+    i = RECVorREAD(CCchan->fd, buff, HEADER_SIZE) ;
+    if (i < 0) {
+	syslog(L_ERROR, "%s cant read CCreader header %m", LogName);
+	return;
+    } else if (i == 0) {
+	syslog(L_ERROR, "%s cant read CCreader header empty", LogName);
+	return;
+    } else if (i != HEADER_SIZE) {
+	syslog(L_ERROR, "%s cant read CCreader header-length %m", LogName);
+	return;
+    }
+
+    memcpy (&protocol,buff,sizeof (protocol)) ;
+    memcpy (&bufflen,buff + sizeof (protocol),sizeof (bufflen)) ;
+    bufflen = ntohs (bufflen) - HEADER_SIZE ;
+    
+    i = RECVorREAD(CCchan->fd, buff, bufflen) ;
+
+    if (i < 0) {
+	syslog(L_ERROR, "%s cant read CCreader buffer %m", LogName);
+	return;
+    } else if (i == 0) {
+	syslog(L_ERROR, "%s cant read CCreader buffer empty", LogName);
+	return;
+    } else if (i != bufflen) {
+	syslog(L_ERROR, "%s cant read CCreader buffer-length %m", LogName);
+	return;
+    }
+
+    buff[i] = '\0';
+
+    if (protocol != ICC_PROTOCOL_1) {
+        syslog(L_ERROR, "%s CCreader protocol mismatch", LogName) ;
+        return ;
+    }
+
+#endif /* defined (DO_HAVE_UNIX_DOMAIN) */
+    
     /* Copy to a printable buffer, and log. */
     (void)strcpy(copy, buff);
     for (p = NULL, q = copy; *q; q++)
@@ -1247,55 +1681,63 @@ CCreader(cp)
     i = *p;
 
     /* Dispatch to the command function. */
-    FreeResult = FALSE;
     for (argc -= 2, dp = CCcommands; dp < ENDOF(CCcommands); dp++)
 	if (i == dp->Name) {
 	    if (argc != dp->argc)
 		p = "1 Wrong number of parameters";
-	    else {
+	    else
 		p = (*dp->Function)(&argv[2]);
-		FreeResult = dp->FreeResult;
-	    }
 	    break;
 	}
     if (dp == ENDOF(CCcommands)) {
 	syslog(L_NOTICE, "%s bad_message %c", LogName, i);
 	p = "1 Bad command";
     }
-    else if (p == NULL) {
+    else if (p == NULL)
 	p = "0 Ok";
-	FreeResult = FALSE;
-    }
 
     /* Build the reply address and send the reply. */
-    len = strlen(p);
+    len = strlen(p) + HEADER_SIZE ;
+    tbuff = NEW(char,len + 1);
+    
+    protocol = ICC_PROTOCOL_1 ;
+    memcpy (tbuff,&protocol,sizeof (protocol)) ;
+    tbuff += sizeof (protocol) ;
+    
+    bufflen = htons (len) ;
+    memcpy (tbuff,&bufflen,sizeof (bufflen)) ;
+    tbuff += sizeof (bufflen) ;
+
+    strcpy (tbuff,p) ;
+    tbuff -= HEADER_SIZE ;
 
 #if	defined(DO_HAVE_UNIX_DOMAIN)
     (void)memset((POINTER)&client, 0, sizeof client);
     client.sun_family = AF_UNIX;
     (void)strcpy(client.sun_path, argv[0]);
-    if (sendto(CCchan->fd, p, len, 0,
+    if (sendto(CCwriter, tbuff, len, 0,
 	    (struct sockaddr *)&client, AF_UNIX_SOCKSIZE(client)) < 0) {
 	i = errno;
 	syslog(i == ENOENT ? L_NOTICE : L_ERROR,
 	    "%s cant sendto CCreader bytes %d %m", LogName, len);
 	if (i == EMSGSIZE)
-	    (void)sendto(CCchan->fd, TOOLONG, STRLEN(TOOLONG), 0,
+	    (void)sendto(CCwriter, TOOLONG, STRLEN(TOOLONG), 0,
 		(struct sockaddr *)&client, AF_UNIX_SOCKSIZE(client));
     }
 #else
-    if ((i = open(argv[0], O_WRONLY)) < 0)
+    if ((i = open(argv[0], O_WRONLY | O_NDELAY)) < 0)
 	syslog(L_ERROR, "%s cant open %s %m", LogName, argv[0]);
     else {
-	if (write(i, p, len) != len)
-	    syslog(L_ERROR, "%s cant write %s %m", LogName, argv[0]);
+	if ((written = write(i, tbuff, len)) != len)
+	    if (written < 0)
+		syslog(L_ERROR, "%s cant write %s %m", LogName, argv[0]);
+	    else
+		syslog(L_ERROR, "%s cant write %s", LogName, argv[0]);
 	if (close(i) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, argv[0]);
     }
 #endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
-
-    if (FreeResult)
-	DISPOSE(p);
+    DISPOSE (tbuff) ;
 }
 
 
@@ -1339,6 +1781,12 @@ CCsetup()
 	syslog(L_FATAL, "%s cant bind %s %m", LogName, CCpath);
 	exit(1);
     }
+
+    /* Create an unbound socket to reply on. */
+    if ((CCwriter = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0) {
+	syslog(L_FATAL, "%s cant socket unbound %m", LogName);
+	exit(1);
+    }
 #else
     /* Create a named pipe and open it. */
     if (mkfifo(CCpath, 0666) < 0) {
@@ -1354,6 +1802,19 @@ CCsetup()
     CCchan = CHANcreate(i, CTcontrol, CSwaiting, CCreader, CCwritedone);
     syslog(L_NOTICE, "%s ccsetup %s", LogName, CHANname(CCchan));
     RCHANadd(CCchan);
+
+    if (CCreply.Size == 0) {
+	CCreply.Size = SMBUF;
+	CCreply.Data = NEW(char, CCreply.Size);
+    }
+
+    /*
+     *  Catch SIGUSR1 so that we can recreate the control channel when
+     *  needed (i.e. something has deleted our named socket.
+     */
+#if     defined(SIGUSR1)
+    (void)signal(SIGUSR1, CCresetup);
+#endif  /* defined(SIGUSR1) */
 }
 
 
@@ -1367,4 +1828,21 @@ CCclose()
     CCchan = NULL;
     if (unlink(CCpath) < 0)
 	syslog(L_ERROR, "%s cant unlink %s %m", LogName, CCpath);
+#if	defined(DO_HAVE_UNIX_DOMAIN)
+    if (close(CCwriter) < 0)
+	syslog(L_ERROR, "%s cant close unbound %m", LogName);
+#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
+}
+
+
+/*
+**  Restablish the control channel.
+*/
+STATIC SIGHANDLER
+CCresetup(s)
+    int         s;
+{
+    (void)signal(s, CCresetup);
+    CCclose();
+    CCsetup();
 }
