@@ -1,21 +1,36 @@
-/*  $Revision: 1.27 $
+/*  $Revision: 1.39 $
 **
 **  I/O channel (and buffer) processing.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 
+/* Divisor of the BUFFER size. If the amount free at the beginning of the
+   buffer is bigger than the quotient, then it is compacted in the
+   readloop */
+#define COMP_THRESHOLD 10
 
 STATIC FDSET	RCHANmask;
 STATIC FDSET	SCHANmask;
 STATIC FDSET	WCHANmask;
 STATIC int	SCHANcount;
 STATIC int	CHANlastfd;
+STATIC int	CHANlastsleepfd;
 STATIC int	CHANccfd;
 STATIC int	CHANtablesize;
 STATIC CHANNEL	*CHANtable;
 STATIC CHANNEL	*CHANcc;
 STATIC CHANNEL	CHANnull = { CTfree, CSerror, -1 };
 
+#define PRIORITISE_REMCONN
+#ifdef PRIORITISE_REMCONN
+STATIC int	CHANrcfd;
+STATIC CHANNEL	*CHANrc;
+#endif /* PRIORITISE_REMCONN */
+
 
 /*
 **  Set a buffer's contents, ignoring anything that might have
@@ -29,15 +44,16 @@ BUFFset(bp, p, length)
 {
     register char	*dest;
 
-    if (length) {
+    if ((bp->Left = length) != 0) {
 	/* Need more space? */
 	if (bp->Size < length) {
 	    bp->Size = GROW_AMOUNT(length);
 	    RENEW(bp->Data, char, bp->Size);
 	}
-	bp->Left = length;
 
-	if (length > MEMCPY_THRESHOLD)
+	/* Try to test for non-overlapping copies. */
+	if (length > MEMCPY_THRESHOLD
+	 && (p < bp->Data || p >= &bp->Data[length]))
 	    (void)memcpy((POINTER)bp->Data, (POINTER)p, (SIZE_T)length);
 	else {
 	    for (dest = bp->Data, length++; --length > 0; )
@@ -48,6 +64,22 @@ BUFFset(bp, p, length)
 }
 
 
+/*
+**  Swap the contents of two buffers.
+*/
+void
+BUFFswap(b1, b2)
+    register BUFFER	*b1;
+    register BUFFER	*b2;
+{
+    BUFFER		b;
+
+    b = *b1;
+    *b1 = *b2;
+    *b2 = b;
+}
+
+
 /*
 **  Initialize all the I/O channels.
 */
@@ -63,9 +95,12 @@ CHANsetup(i)
     if (CHANtable)
 	DISPOSE(CHANtable);
     CHANtablesize = i;
+    CHANtable = NEW(CHANNEL, CHANtablesize);
+    (void)memset((POINTER)CHANtable, 0,
+	    (SIZE_T)(CHANtablesize * sizeof *CHANtable));
     CHANnull.NextLog = CHANNEL_INACTIVE_TIME;
     CHANnull.Address.s_addr = MyAddress.s_addr;
-    for (cp = CHANtable = NEW(CHANNEL, CHANtablesize); --i >= 0; cp++)
+    for (cp = CHANtable; --i >= 0; cp++)
 	*cp = CHANnull;
 }
 
@@ -108,23 +143,40 @@ CHANcreate(fd, Type, State, Reader, WriteDone)
     cp->fd = fd;
     cp->Type = Type;
     cp->State = State;
+    cp->Streaming = FALSE ;
     cp->Reader = Reader;
     cp->WriteDone = WriteDone;
     cp->Started = cp->LastActive = Now.time;
     cp->In = in;
     cp->Out = out;
     cp->Tracing = Tracing;
+    cp->Sendid.Size=0;
+    cp->Rest=0;
+    cp->SaveUsed=0;
+    cp->Lastch=0;
 
     /* Make the descriptor close-on-exec and non-blocking. */
-    CloseOnExec(fd);
+    CloseOnExec(fd, TRUE);
+#if	defined(ENOTSOCK)
+    if (SetNonBlocking(fd, TRUE) < 0 && errno != ENOTSOCK)
+	syslog(L_ERROR, "%s cant nonblock %d %m", LogName, fd);
+#else
     if (SetNonBlocking(fd, TRUE) < 0)
 	syslog(L_ERROR, "%s cant nonblock %d %m", LogName, fd);
+#endif	/* defined(ENOTSOCK) */
 
     /* Note control channel, for efficiency. */
     if (Type == CTcontrol) {
 	CHANcc = cp;
 	CHANccfd = fd;
     }
+#ifdef PRIORITISE_REMCONN
+    /* Note remconn channel, for efficiency */
+    if (Type == CTremconn) {
+	CHANrc = cp;
+	CHANrcfd = fd;
+    }
+#endif /* PRIORITISE_REMCONN */
     return cp;
 }
 
@@ -144,18 +196,18 @@ CHANtracing(cp, Flag)
     cp->Tracing = Flag;
     if (Flag) {
 	syslog(L_NOTICE, "%s trace badwrites %d blockwrites %d badreads %d",
-	    p, cp->BadWrites, cp->BadReads, cp->BlockedWrites);
-	syslog(L_NOTICE, "%s trace address %s lastactive %ld nextlod %ld",
+	    p, cp->BadWrites, cp->BlockedWrites, cp->BadReads);
+	syslog(L_NOTICE, "%s trace address %s lastactive %ld nextlog %ld",
 	    p, inet_ntoa(cp->Address), cp->LastActive, cp->NextLog);
 	if (FD_ISSET(cp->fd, &SCHANmask))
 	    syslog(L_NOTICE, "%s trace sleeping %ld 0x%x",
 		p, (long)cp->Waketime, cp->Waker);
 	if (FD_ISSET(cp->fd, &RCHANmask))
-	    syslog(L_NOTICE, "%s trace reading %d %.45s",
-		p, cp->In.Used, cp->In.Data);
+	    syslog(L_NOTICE, "%s trace reading %d %s",
+		p, cp->In.Used, MaxLength(cp->In.Data, cp->In.Data));
 	if (FD_ISSET(cp->fd, &WCHANmask))
-	    syslog(L_NOTICE, "%s trace writing %d %.45s",
-		p, cp->Out.Left, cp->Out.Data);
+	    syslog(L_NOTICE, "%s trace writing %d %s",
+		p, cp->Out.Left, MaxLength(cp->Out.Data, cp->Out.Data));
     }
 }
 
@@ -169,18 +221,25 @@ CHANclose(cp, name)
     char		*name;
 {
     if (cp->Type == CTfree)
-	syslog(L_ERROR, "%s internal closing free channel", name);
+	syslog(L_ERROR, "%s internal closing free channel %d", name, cp->fd);
     else {
 	if (cp->Type == CTnntp)
 	    syslog(L_NOTICE,
 		"%s closed seconds %ld accepted %ld refused %ld rejected %ld",
 		name, (long)(Now.time - cp->Started),
 		cp->Received, cp->Refused, cp->Rejected);
+	else if (cp->Type == CTreject)
+	    syslog(L_NOTICE, "%s %ld", name, cp->Rejected);
+	else if (cp->Out.Left)
+	    syslog(L_NOTICE, "%s closed lost %d", name, cp->Out.Left);
 	else
 	    syslog(L_NOTICE, "%s closed", name);
 	WCHANremove(cp);
 	RCHANremove(cp);
 	SCHANremove(cp);
+	if (cp->Argument != NULL)
+	    /* Set to NULL below. */
+	    DISPOSE(cp->Argument);
 	if (cp->fd >= 0 && close(cp->fd) < 0)
 	    syslog(L_ERROR, "%s cant close %s %m", LogName, name);
     }
@@ -189,6 +248,7 @@ CHANclose(cp, name)
     cp->Type = CTfree;
     cp->State = CSerror;
     cp->fd = -1;
+    cp->Argument = NULL;
 
     /* Free the buffers if they got big. */
     if (cp->In.Size > BIG_BUFFER) {
@@ -199,6 +259,10 @@ CHANclose(cp, name)
 	cp->Out.Size = 0;
 	DISPOSE(cp->Out.Data);
     }
+    if (cp->Sendid.Size) {
+	cp->Sendid.Size = 0;
+	DISPOSE(cp->Sendid.Data);
+    }
 }
 
 
@@ -228,6 +292,9 @@ CHANname(cp)
     case CTremconn:
 	(void)sprintf(buff, "remconn:%d", cp->fd);
 	break;
+    case CTreject:
+	(void)sprintf(buff, "%s rejected", RChostname(cp));
+	break;
     case CTnntp:
 	(void)sprintf(buff, "%s:%d",
 		cp->Address.s_addr == 0 ? "localhost" : RChostname(cp),
@@ -251,11 +318,13 @@ CHANname(cp)
 		break;
 	    }
 	if (pid == 0)
-	    (void)sprintf(buff, "%.45s:%d:%s",
-		    p, cp->fd, cp->Type == CTfile ? "file" : "proc");
+	    (void)sprintf(buff, "%s:%d:%s",
+		    MaxLength(p, p), cp->fd,
+		    cp->Type == CTfile ? "file" : "proc");
 	else
-	    (void)sprintf(buff, "%.45s:%d:%s:%ld",
-		    p, cp->fd, cp->Type == CTfile ? "file" : "proc", (long)pid);
+	    (void)sprintf(buff, "%s:%d:%s:%ld",
+		    MaxLength(p, p), cp->fd,
+		    cp->Type == CTfile ? "file" : "proc", (long)pid);
 	break;
     }
     return buff;
@@ -289,6 +358,8 @@ CHANiter(ip, Type)
     if ((i = *ip) >= 0 && i < CHANtablesize) {
 	do {
 	    cp = &CHANtable[i];
+	    if (cp->Type == CTfree && cp->fd == -1)
+		continue;
 	    if (Type == CTany || cp->Type == Type) {
 		*ip = ++i;
 		return cp;
@@ -337,21 +408,29 @@ RCHANremove(cp)
 
 /*
 **  Put a channel to sleep, call a function when it wakes.
+**  Note that the Argument must be NULL or allocated memory!
 */
 void
-SCHANadd(cp, Waketime, Waker, Argument)
+SCHANadd(cp, Waketime, Event, Waker, Argument)
     register CHANNEL	*cp;
     time_t		Waketime;
+    POINTER		Event;
     FUNCPTR		Waker;
-    char		*Argument;
+    POINTER		Argument;
 {
     if (!FD_ISSET(cp->fd, &SCHANmask)) {
 	SCHANcount++;
 	FD_SET(cp->fd, &SCHANmask);
     }
+    if (cp->fd > CHANlastsleepfd)
+	CHANlastsleepfd = cp->fd;
     cp->Waketime = Waketime;
     cp->Waker = Waker;
-    cp->Argument = Argument;
+    if (cp->Argument != Argument) {
+	DISPOSE(cp->Argument);
+	cp->Argument = Argument;
+    }
+    cp->Event = Event;
 }
 
 
@@ -366,6 +445,12 @@ SCHANremove(cp)
 	FD_CLR(cp->fd, &SCHANmask);
 	SCHANcount--;
 	cp->Waketime = 0;
+	if (cp->fd == CHANlastsleepfd) {
+	    /* This was the highest descriptor, get a new highest. */
+	    while (!FD_ISSET(CHANlastsleepfd, &SCHANmask)
+	      && CHANlastsleepfd > 1)
+		CHANlastsleepfd--;
+	}
     }
 }
 
@@ -381,6 +466,22 @@ CHANsleeping(cp)
 }
 
 
+/*
+**  Wake up channels waiting for a specific event.
+*/
+void
+SCHANwakeup(Event)
+    register POINTER	Event;
+{
+    register CHANNEL	*cp;
+    register int	i;
+
+    for (cp = CHANtable, i = CHANtablesize; --i >= 0; cp++)
+	if (cp->Type != CTfree && cp->Event == Event && CHANsleeping(cp))
+	    cp->Waketime = 0;
+}
+
+
 /*
 **  Mark a channel as an active writer.  Don't reset the Out->Left field
 **  since we could have buffered I/O already in there.
@@ -445,32 +546,57 @@ CHANreadtext(cp)
     register int	i;
     register BUFFER	*bp;
     char		*p;
+    int			oerrno;
 
-    p = CHANname(cp);
-
-    /* Read in whatever is there. */
+    /* Grow buffer if we're getting close to current limit. */
     bp = &cp->In;
-    bp->Left = bp->Size - bp->Used;
-    i = read(cp->fd, &bp->Data[bp->Used], bp->Left - 1);
-    if (i < 0) {
-	syslog(L_ERROR, "%s cant read %m", p);
-	return -1;
-    }
-    if (i == 0) {
-	syslog(L_NOTICE, "%s readclose", p);
-	CHANclose(cp, p);
-	return 0;
-    }
-
-    /* Update values, grow buffer if we're getting close. */
-    bp->Used += i;
-    bp->Left -= i;
     if (bp->Left <= LOW_WATER) {
 	i = GROW_AMOUNT(bp->Size);
 	bp->Size += i;
 	bp->Left += i;
 	RENEW(bp->Data, char, bp->Size);
     }
+
+    /* Read in whatever is there, up to some reasonable limit. */
+    /*
+     * XXX We really want to limit the amount of time it takes to
+     * process the incoming data for this channel.  But there's
+     * no easy way of doing that, so we restrict the data size instead.
+     * If the data is part of a single large article, then reading
+     * and processing many kilobytes at a time costs very little.
+     * If the data is a long list of CHECK commands from a streaming
+     * feed, then every line of data will require a history lookup, and
+     * we probably don't want to do more than about 10 of those per
+     * channel on each cycle of the main select() loop (otherwise we
+     * might take too long before giving other channels a turn).  10
+     * lines of CHECK commands suggests a limit of about 1 kilobyte of
+     * data, or less.  BUFSIZ is often about 1 kilobyte, and is
+     * attractive for other reasons, so let's use that as our size limit.
+     */
+    bp->Left = bp->Size - bp->Used;
+    i = read(cp->fd, &bp->Data[bp->Used],
+	      (bp->Left - 1 > BUFSIZ ? BUFSIZ : bp->Left - 1));
+    if (i < 0) {
+#ifdef POLL_BUG
+    /* return of -2 indicates EAGAIN, for SUNOS5.4 poll() bug workaround */
+        if (errno == EAGAIN) {
+            return -2;
+        }
+#endif
+	oerrno = errno;
+	p = CHANname(cp);
+	errno = oerrno;
+	syslog(L_ERROR, "%s cant read %m", p);
+	return -1;
+    }
+    if (i == 0) {
+	p = CHANname(cp);
+	syslog(L_NOTICE, "%s readclose", p);
+	return 0;
+    }
+
+    bp->Used += i;
+    bp->Left -= i;
     return i;
 }
 
@@ -489,11 +615,14 @@ largewrite(fd, p, length)
     register int	i;
     register char	*save;
 
-    /* Try the standard case -- write it all. */
-    i = write(fd, (POINTER)p, (SIZE_T)length);
-    if (i > 0 || errno != EMSGSIZE)
-	return i;
+    do {
+	/* Try the standard case -- write it all. */
+	i = write(fd, (POINTER)p, (SIZE_T)length);
+	if (i > 0 || (i < 0 && errno != EMSGSIZE && errno != EINTR))
+	    return i;
+    } while (i < 0 && errno == EINTR);
 
+    /* Write it in pieces. */
     for (save = p, i = 0; length; p += i, length -= i) {
 	i = write(fd, (POINTER)p, (SIZE_T)(length > BUFSIZ ? BUFSIZ : length));
 	if (i <= 0)
@@ -515,24 +644,21 @@ WCHANflush(cp)
     register BUFFER	*bp;
     register int	i;
 
-    /* If nothing in there, or nothing left, nothing to do. */
-    bp = &cp->Out;
-    if (bp->Left == 0)
-	return TRUE;
-
     /* Write it. */
-    while (bp->Left > 0) {
-	if ((i = write(cp->fd, (POINTER)&bp->Data[bp->Used],
-		    (SIZE_T)bp->Left)) <= 0) {
+    for (bp = &cp->Out; bp->Left > 0; bp->Left -= i, bp->Used += i) {
+	i = largewrite(cp->fd, &bp->Data[bp->Used], bp->Left);
+	if (i < 0) {
 	    syslog(L_ERROR, "%s cant flush count %d %m",
 		CHANname(cp), bp->Left);
 	    return FALSE;
 	}
-	bp->Left -= i;
-	bp->Used += i;
-	if (bp->Left <= 0)
-	    WCHANremove(cp);
+	if (i == 0) {
+	    syslog(L_ERROR, "%s cant flush count %d",
+		CHANname(cp), bp->Left);
+	    return FALSE;
+	}
     }
+    WCHANremove(cp);
     return TRUE;
 }
 
@@ -541,11 +667,9 @@ WCHANflush(cp)
 /*
 **  Wakeup routine called after a write channel was put to sleep.
 */
-/* ARGSUSED1 */
 STATIC FUNCTYPE
-CHANwakeup(cp, Argument)
+CHANwakeup(cp)
     CHANNEL	*cp;
-    char	*Argument;
 {
     syslog(L_NOTICE, "%s wakeup", CHANname(cp));
     WCHANadd(cp);
@@ -566,6 +690,7 @@ CHANwritesleep(cp, p)
 	switch (cp->Type) {
 	default:
 	    break;
+	case CTreject:
 	case CTnntp:
 	case CTfile:
 	case CTexploder:
@@ -577,19 +702,68 @@ CHANwritesleep(cp, p)
 	}
     i *= BLOCK_BACKOFF;
     syslog(L_ERROR, "%s blocked sleeping %d", p, i);
-    SCHANadd(cp, (time_t)(Now.time + i), CHANwakeup, (char *)NULL);
+    SCHANadd(cp, (time_t)(Now.time + i), (POINTER)NULL,
+	CHANwakeup, (POINTER)NULL);
 }
 
 
+#if	defined(INND_FIND_BAD_FDS)
+/*
+**  We got an unknown error in select.  Find out the culprit.
+**  Not really ready for production use yet, and it's expensive, too.
+*/
+STATIC void
+CHANdiagnose()
+{
+    FDSET		Test;
+    int			i;
+    struct timeval	t;
+
+    FD_ZERO(&Test);
+    for (i = CHANlastfd; i >= 0; i--) {
+	if (FD_ISSET(i, &RCHANmask)) {
+	    FD_SET(i, &Test);
+	    t.tv_sec = 0;
+	    t.tv_usec = 0;
+	    if (select(i + 1, &Test, (FDSET *)NULL, (FDSET *)NULL, &t) < 0
+	      && errno != EINTR) {
+		syslog(L_ERROR, "%s Bad Read File %d", LogName, i);
+		FD_CLR(i, &RCHANmask);
+		/* Probably do something about the file descriptor here; call
+		 * CHANclose on it? */
+	    }
+	    FD_CLR(i, &Test);
+	}
+	if (FD_ISSET(i, &WCHANmask)) {
+	    FD_SET(i, &Test);
+	    t.tv_sec = 0;
+	    t.tv_usec = 0;
+	    if (select(i + 1, (FDSET *)NULL, &Test, (FDSET *)NULL, &t) < 0
+	     && errno != EINTR) {
+		syslog(L_ERROR, "%s Bad Write File %d", LogName, i);
+		FD_CLR(i, &WCHANmask);
+		/* Probably do something about the file descriptor here; call
+		 * CHANclose on it? */
+	    }
+	    FD_CLR(i, &Test);
+	}
+    }
+}
+#endif	/* defined(INND_FIND_BAD_FDS) */
+
+
 /*
 **  Main I/O loop.  Wait for data, call the channel's handler when there is
 **  something to read or when the queued write is finished.  In order to
 **  be fair (i.e., don't always give descriptor n priority over n+1), we
 **  remember where we last had something and pick up from there.
+**
+**  Yes, the main code has really wandered over to the side a lot.
 */
 void
 CHANreadloop()
 {
+    static char		EXITING[] = "INND exiting because of signal\n";
     static int		fd;
     register int	i;
     register int	startpoint;
@@ -603,7 +777,9 @@ CHANreadloop()
     struct timeval	MyTime;
     long		silence;
     char		*p;
+    time_t		LastUpdate;
 
+    LastUpdate = GetTimeInfo(&Now) < 0 ? 0 : Now.time;
     for ( ; ; ) {
 	/* See if any processes died. */
 	PROCscan();
@@ -615,15 +791,31 @@ CHANreadloop()
 	count = select(CHANlastfd + 1, &MyRead, &MyWrite, (FDSET *)NULL,
 		&MyTime);
 	if (GotTerminate) {
-	    (void)fputs("INND exiting because of SIGTERM\n", stderr);
-	    CleanupAndExit(0, "received signal");
+	    (void)write(2, EXITING, STRLEN(EXITING));
+	    CleanupAndExit(0, (char *)NULL);
 	}
 	if (count < 0) {
-	    if (errno != EINTR)
+	    if (errno != EINTR) {
 		syslog(L_ERROR, "%s cant select %m", LogName);
+#if	defined(INND_FIND_BAD_FDS)
+		CHANdiagnose();
+#endif	/* defined(INND_FIND_BAD_FDS) */
+	    }
 	    continue;
 	}
 
+	/* Update the "reasonably accurate" time. */
+	if (GetTimeInfo(&Now) < 0)
+	    syslog(L_ERROR, "%s cant gettimeinfo %m", LogName);
+	if (Now.time > LastUpdate + TimeOut.tv_sec) {
+	    HISsync();
+	    if (ICDactivedirty) {
+		ICDwriteactive();
+		ICDactivedirty = 0;
+	    }
+            LastUpdate = Now.time;
+	}
+
 	if (count == 0) {
 	    /* No channels active, so flush and skip if nobody's
 	     * sleeping. */
@@ -633,17 +825,24 @@ CHANreadloop()
 		continue;
 	}
 
-	/* Update the "reasonably accurate" time. */
-	if (GetTimeInfo(&Now) < 0)
-	    syslog(L_ERROR, "%s cant gettimeinfo %m", LogName);
-
 	/* Try the control channel first. */
 	if (FD_ISSET(CHANccfd, &RCHANmask) && FD_ISSET(CHANccfd, &MyRead)) {
 	    count--;
+ 	    if (count > 3) count = 3; /* might be more requests */
 	    (*CHANcc->Reader)(CHANcc);
 	    FD_CLR(CHANccfd, &MyRead);
 	}
 
+#ifdef PRIORITISE_REMCONN
+	/* Try the remconn channel next. */
+	if (FD_ISSET(CHANrcfd, &RCHANmask) && FD_ISSET(CHANrcfd, &MyRead)) {
+	    count--;
+	    if (count > 3) count = 3; /* might be more requests */
+	    (*CHANrc->Reader)(CHANrc);
+	    FD_CLR(CHANrcfd, &MyRead);
+	}
+#endif /* PRIORITISE_REMCONN */
+
 	/* Loop through all active channels.  Somebody could have closed
 	 * closed a channel so we double-check the global mask before
 	 * looking at what select returned.  The code here is written so
@@ -651,19 +850,28 @@ CHANreadloop()
 	 * same time, even though that's not possible.  (Just as well,
 	 * since in SysVr4 the count would be wrong.) */
 	lastfd = CHANlastfd;
+	if (lastfd < CHANlastsleepfd)
+	    lastfd = CHANlastsleepfd;
 	if (fd > lastfd)
 	    fd = 0;
 	startpoint = fd;
 	do {
 	    cp = &CHANtable[fd];
-	    if (cp->Type == CTfree)
-		goto Next;
 
 	    /* Anything to read? */
 	    if (FD_ISSET(fd, &RCHANmask) && FD_ISSET(fd, &MyRead)) {
 		count--;
-		cp->LastActive = Now.time;
-		(*cp->Reader)(cp);
+		if (cp->Type == CTfree) {
+		    syslog(L_ERROR, "%s %d free but was in RMASK",
+			CHANname(cp), fd);
+		    /* Don't call RCHANremove since cp->fd will be -1. */
+		    FD_CLR(fd, &RCHANmask);
+		    (void)close(fd);
+		}
+		else {
+		    cp->LastActive = Now.time;
+		    (*cp->Reader)(cp);
+		}
 	    }
 
 	    /* Possibly recheck for dead children so we don't get SIGPIPE
@@ -674,48 +882,84 @@ CHANreadloop()
 	    /* Ready to write? */
 	    if (FD_ISSET(fd, &WCHANmask) && FD_ISSET(fd, &MyWrite)) {
 		count--;
-		bp = &cp->Out;
-		if (bp->Left) {
-		    cp->LastActive = Now.time;
-		    i = largewrite(fd, &bp->Data[bp->Used], bp->Left);
-		    if (i <= 0) {
-			p = CHANname(cp);
-			oerrno = errno;
-			syslog(L_ERROR, "%s cant write %m", p);
-			cp->BadWrites++;
-			if (i < 0 && oerrno == EPIPE) {
-			    SITEchanclose(cp);
-			    CHANclose(cp, p);
+		if (cp->Type == CTfree) {
+		    syslog(L_ERROR, "%s %d free but was in WMASK",
+			CHANname(cp), fd);
+		    /* Don't call WCHANremove since cp->fd will be -1. */
+		    FD_CLR(fd, &WCHANmask);
+		    (void)close(fd);
+		}
+		else {
+		    bp = &cp->Out;
+		    if (bp->Left) {
+			cp->LastActive = Now.time;
+			i = largewrite(fd, &bp->Data[bp->Used], bp->Left);
+			if (i <= 0) {
+			    oerrno = errno;
+			    p = CHANname(cp);
+			    errno = oerrno;
+			    if (i < 0)
+				syslog(L_ERROR, "%s cant write %m", p);
+			    else
+				syslog(L_ERROR, "%s cant write", p);
+			    cp->BadWrites++;
+			    if (i < 0 && oerrno == EPIPE) {
+				SITEchanclose(cp);
+				CHANclose(cp, p);
+			    }
+			    else if (i < 0 && oerrno == EWOULDBLOCK) {
+				WCHANremove(cp);
+				CHANwritesleep(cp, p);
+			    }
+			    else if (cp->BadWrites >= BAD_IO_COUNT) {
+				syslog(L_ERROR, "%s sleeping", p);
+				WCHANremove(cp);
+				SCHANadd(cp,
+				    (time_t)(Now.time + PAUSE_RETRY_TIME),
+				    (POINTER)NULL, CHANwakeup, (POINTER)NULL);
+			    }
 			}
-			else if (oerrno == EWOULDBLOCK) {
-			    WCHANremove(cp);
-			    CHANwritesleep(cp, p);
-			}
-			else if (cp->BadWrites >= BAD_IO_COUNT) {
-			    syslog(L_ERROR, "%s sleeping", p);
-			    WCHANremove(cp);
-			    SCHANadd(cp, (time_t)(Now.time + PAUSE_RETRY_TIME),
-				CHANwakeup, (char *)NULL);
-			}
-		    }
-		    else {
-			cp->BadWrites = 0;
-			cp->BlockedWrites = 0;
-			bp->Left -= i;
-			bp->Used += i;
-			if (bp->Left <= 0) {
-			    WCHANremove(cp);
-			    (*cp->WriteDone)(cp);
+			else {
+			    cp->BadWrites = 0;
+			    cp->BlockedWrites = 0;
+			    bp->Left -= i;
+			    bp->Used += i;
+			    if (bp->Left <= 0) {
+				WCHANremove(cp);
+				(*cp->WriteDone)(cp);
+			    } else if (bp->Used > (bp->Size/COMP_THRESHOLD)) {
+                                /* compact the buffer, shoving the
+                                   data back to the beginning.
+                                   <rmtodd@mailhost.ecn.ou.edu> */
+                                BUFFset(bp, &bp->Data[bp->Used], bp->Left);
+ 			    }
 			}
 		    }
+		    else
+			/* Should not be possible. */
+			WCHANremove(cp);
 		}
 	    }
 
 	    /* Coming off a sleep? */
 	    if (FD_ISSET(fd, &SCHANmask) && cp->Waketime <= Now.time) {
-		cp->LastActive = Now.time;
-		SCHANremove(cp);
-		(*cp->Waker)(cp, cp->Argument);
+		if (cp->Type == CTfree) {
+		    syslog(L_ERROR,"%s ERROR s-select free %d",CHANname(cp),fd);
+		    FD_CLR(fd, &SCHANmask);
+		    (void) close(fd);
+		} else {
+		    cp->LastActive = Now.time;
+		    SCHANremove(cp);
+		    (*cp->Waker)(cp);
+		}
+	    }
+
+	    /* Toss CTreject channel early if it's inactive. */
+	    if (cp->Type == CTreject
+	     && cp->LastActive + REJECT_TIMEOUT < Now.time) {
+		p = CHANname(cp);
+		syslog(L_NOTICE, "%s timeout reject", p);
+		CHANclose(cp, p);
 	    }
 
 	    /* Has this channel been inactive very long? */
@@ -731,7 +975,6 @@ CHANreadloop()
 		}
 	    }
 
-    Next:
 	    /* Bump pointer, modulo the table size. */
 	    if (fd >= lastfd)
 		fd = 0;
