@@ -1,7 +1,11 @@
-/*  $Revision: 1.18 $
+/*  $Revision: 1.23 $
 **
 **  History file routines.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 #include "dbz.h"
 
@@ -10,6 +14,7 @@ STATIC char	HIShistpath[] = _PATH_HISTORY;
 STATIC FILE	*HISwritefp;
 STATIC int	HISreadfd;
 STATIC int	HISdirty;
+STATIC int	HISincore = INND_DBZINCORE;
 
 
 /*
@@ -20,23 +25,24 @@ HISsetup()
 {
     if (HISwritefp == NULL) {
 	/* Open the history file for appending formatted I/O. */
-	if ((HISwritefp = xfopena(HIShistpath)) == NULL) {
+	if ((HISwritefp = fopen(HIShistpath, "a")) == NULL) {
 	    syslog(L_FATAL, "%s cant fopen %s %m", LogName, HIShistpath);
 	    exit(1);
 	}
-	CloseOnExec((int)fileno(HISwritefp));
+	CloseOnExec((int)fileno(HISwritefp), TRUE);
 
 	/* Open the history file for reading. */
 	if ((HISreadfd = open(HIShistpath, O_RDONLY)) < 0) {
 	    syslog(L_FATAL, "%s cant open %s %m", LogName, HIShistpath);
 	    exit(1);
 	}
-	CloseOnExec(HISreadfd);
+	CloseOnExec(HISreadfd, TRUE);
 
 	/* Open the DBZ file. */
-	(void)dbzincore(1);
+	(void)dbzincore(HISincore);
+	(void)dbzwritethrough(1);
 	if (dbminit(HIShistpath) < 0) {
-	    syslog(L_FATAL, "%s cant dbminit %m", LogName);
+	    syslog(L_FATAL, "%s cant dbminit %s %m", HIShistpath, LogName);
 	    exit(1);
 	}
     }
@@ -246,19 +252,20 @@ HISwrite(Data, paths)
     if (Data->Expires > 0)
 	i = fprintf(HISwritefp, "%s%c%ld%c%ld%c%ld%c%s\n",
 		key.dptr, HIS_FIELDSEP,
-		(long)Now.time, HIS_SUBFIELDSEP, (long)Data->Expires,
+		(long)Data->Arrived, HIS_SUBFIELDSEP, (long)Data->Expires,
 		    HIS_SUBFIELDSEP, (long)Data->Posted, HIS_FIELDSEP,
 		paths);
     else
 	i = fprintf(HISwritefp, "%s%c%ld%c%s%c%ld%c%s\n",
 		key.dptr, HIS_FIELDSEP,
-		(long)Now.time, HIS_SUBFIELDSEP, HIS_NOEXP,
+		(long)Data->Arrived, HIS_SUBFIELDSEP, HIS_NOEXP,
 		    HIS_SUBFIELDSEP, (long)Data->Posted, HIS_FIELDSEP,
 		paths);
     if (i == EOF || fflush(HISwritefp) == EOF) {
 	/* The history line is now an orphan... */
-	IOError("history");
+	i = errno;
 	syslog(L_ERROR, "%s cant write history %m", LogName);
+	IOError("history", i);
 	return FALSE;
     }
 
@@ -266,8 +273,9 @@ HISwrite(Data, paths)
     val.dptr = (char *)&offset;
     val.dsize = sizeof offset;
     if (dbzstore(key, val) < 0) {
-	IOError("history database");
+	i = errno;
 	syslog(L_ERROR, "%s cant dbzstore %m", LogName);
+	IOError("history database", i);
 	return FALSE;
     }
 
