@@ -1,4 +1,4 @@
-/*  $Revision: 1.50 $
+/*  $Revision: 1.51 $
 **
 **  Routines for the NNTP channel.  Other channels get the descriptors which
 **  we turn into NNTP channels, and over which we speak NNTP.
@@ -1019,6 +1019,9 @@ NCproc(cp)
 	     && (bp->Data[1] == '\r') && (bp->Data[2] == '\n')) {
 		cp->Rest = 3;	/* null article (canceled?) */
 		cp->Rejected++;
+#if 0
+                syslog(L_NOTICE, "%s empty article", CHANname(cp)) ;
+#endif
 		if (cp->Sendid.Size > 3) { /* We be streaming */
 		    char buff[4];
 		    (void)sprintf(buff, "%d", NNTP_ERR_FAILID_VAL);
@@ -1511,7 +1514,7 @@ NCtakethis(cp)
     register int	i;
 
     /* Snip off the Message-ID. */
-    for (p = cp->In.Data; !ISWHITE(*p); p++)
+    for (p = cp->In.Data + STRLEN("takethis"); ISWHITE(*p); p++)
 	continue;
     for ( ; ISWHITE(*p); p++)
 	continue;
