@@ -1,7 +1,11 @@
-/*  $Revision: 1.31 $
+/*  $Revision: 1.38 $
 **
 **  Routines for the in-core data structures for the newsfeeds file.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 
 
@@ -39,7 +43,7 @@ SITEreadfile(ReadOnly)
     static char		**old_strings;
     static time_t	old_mtime;
     static ino_t	old_ino;
-    static off_t	old_size;
+    static OFFSET_T	old_size;
     register char	*p;
     register char	*to;
     register char	*site;
@@ -104,20 +108,27 @@ SITEreadfile(ReadOnly)
 **  Modify "subbed" according to the patterns in "patlist."
 */
 STATIC void
-SITEsetlist(patlist, subbed)
+SITEsetlist(patlist, subbed, poison, poisonEntry)
     char		**patlist;
     char		*subbed;
+    char		*poison;
+    BOOL		*poisonEntry;
 {
     register char	*pat;
     register char	*p;
+    register char	*u;
     register char	subvalue;
+    register char	poisonvalue;
     register NEWSGROUP	*ngp;
     register int	i;
 
     while ((pat = *patlist++) != NULL) {
-	subvalue = *pat != SUB_NEGATE;
+	subvalue = *pat != SUB_NEGATE && *pat != SUB_POISON;
+	poisonvalue = *pat == SUB_POISON;
 	if (!subvalue)
 	    pat++;
+	if (!*pat)
+	    continue;
 
 	/* See if pattern is a simple newsgroup name.  If so, set the
 	 * right subbed element for that one group (if found); if not,
@@ -127,13 +138,18 @@ SITEsetlist(patlist, subbed)
 		break;
 	if (*p == '\0') {
 	    /* Simple string; look it up, set it. */
-	    if ((ngp = NGfind(pat)) != NULL)
+	    if ((ngp = NGfind(pat)) != NULL) {
 		subbed[ngp - Groups] = subvalue;
+		poison[ngp - Groups] = poisonvalue;
+	    }
 	}
 	else
-	    for (p = subbed, ngp = Groups, i = nGroups; --i >= 0; ngp++, p++)
-		if (wildmat(ngp->Name, pat))
+	    for (p = subbed, u = poison, ngp = Groups, i = nGroups;
+			--i >= 0; ngp++, p++, u++)
+		if (wildmat(ngp->Name, pat)) {
 		    *p = subvalue;
+		    *u = poisonvalue;
+		}
     }
 }
 
@@ -146,16 +162,18 @@ SITEsetlist(patlist, subbed)
 **  doing syntax checking.
 */
 STRING
-SITEparseone(Entry, sp, subbed)
+SITEparseone(Entry, sp, subbed, poison)
     char		*Entry;
     SITE		*sp;
     char		*subbed;
+    char		*poison;
 {
     static char		BATCH[] = _PATH_BATCHDIR;
     register int	i;
     register int	j;
     register NEWSGROUP	*ngp;
     register char	*p;
+    register char	*u;
     char		*f2;
     char		*f3;
     char		*f4;
@@ -172,7 +190,9 @@ SITEparseone(Entry, sp, subbed)
     sp->Buffer = b;
     sp->Master = NOSITE;
     sp->Funnel = NOSITE;
+    sp->PoisonEntry = FALSE;
     sp->Process = -1;
+    sp->Next = sp->Prev = NOSITE;
     sp->Entry = Entry;
     sp->FileFlags[0] = FEED_NAME;
     sp->FileFlags[1] = '\0';
@@ -185,6 +205,7 @@ SITEparseone(Entry, sp, subbed)
     if ((p = strchr(sp->Name, NF_SUBFIELD_SEP)) != NULL) {
 	/* Exclusions within the site field. */
 	*p++ = '\0';
+	if (*p)
 	sp->Exclusions = CommaSplit(p);
     }
     sp->NameLength = strlen(sp->Name);
@@ -196,16 +217,19 @@ SITEparseone(Entry, sp, subbed)
     if ((p = strchr(f2, NF_SUBFIELD_SEP)) != NULL) {
 	/* Distributions within the subscription field. */
 	*p++ = '\0';
+	if (*p)
 	sp->Distributions = CommaSplit(p);
     }
+    if (f2)
     sp->Patterns = CommaSplit(f2);
 
     if (subbed) {
 	/* Read the subscription patterns and set the bits. */
 	(void)memset((POINTER)subbed, SUB_DEFAULT, (SIZE_T)nGroups);
+	(void)memset((POINTER)poison, SUB_DEFAULT, (SIZE_T)nGroups);
 	if (ME.Patterns)
-	    SITEsetlist(ME.Patterns, subbed);
-	SITEsetlist(sp->Patterns, subbed);
+	    SITEsetlist(ME.Patterns, subbed, poison, &ME.PoisonEntry);
+	SITEsetlist(sp->Patterns, subbed, poison, &sp->PoisonEntry);
     }
 
     /* Get the third field, the flags. */
@@ -225,6 +249,10 @@ SITEparseone(Entry, sp, subbed)
 	    if (*++p && CTYPE(isdigit, *p))
 		sp->MaxSize = atol(p);
 	    break;
+	case '>':
+	    if (*++p && CTYPE(isdigit, *p))
+		sp->MinSize = atol(p);
+	    break;
 	case 'A':
 	    while (*++p)
 		switch (*p) {
@@ -244,7 +272,9 @@ SITEparseone(Entry, sp, subbed)
 	    }
 	    break;
 	case 'F':
-	    if (*++p == '/')
+	    if (*++p == '\0')
+		return "missing file name for F param in field 3";
+	    else if (*p == '/')
 		sp->SpoolName = COPY(p);
 	    else {
 		sp->SpoolName = NEW(char, STRLEN(BATCH) + 1 + strlen(p) + 1);
@@ -316,6 +346,7 @@ SITEparseone(Entry, sp, subbed)
 		case FEED_REPLIC:		/* Replication data	*/
 		case FEED_SITE:			/* Site that gave it	*/
 		case FEED_TIMERECEIVED:		/* When received	*/
+		case FEED_TIMEPOSTED:		/* When posted		*/
 		    break;
 		}
 		sp->FileFlags[i++] = *p;
@@ -328,6 +359,8 @@ SITEparseone(Entry, sp, subbed)
     DISPOSE(save);
     if (sp->Flushpoint && sp->Type != FTfile)
 	return "I param with non-file feed";
+    if (sp->Flushpoint == 0 && sp->Type == FTfile)
+	sp->Flushpoint = SITE_BUFFER_SIZE;
 
     if (subbed) {
 	/* Modify the subscription list based on the flags. */
@@ -362,10 +395,21 @@ SITEparseone(Entry, sp, subbed)
 
     /* Make sure there is only one %s, and only one *. */
     if (sp->Type == FTprogram) {
-	if ((p = strchr(sp->Param, '%')) != NULL) {
+	f3 = NULL;
+	for (f2 = sp->Param; *f2; f2 = p + 1) {
+	    p = strchr(f2, '%');
+	    if (p == NULL)
+		break;
+	    if (p[1] == '%') {
+		p++;
+		continue;
+	    }
+	    if (f3 != NULL)
+		return "bad (extra) sprintf format for field 4";
+	    f3 = p;
 	    while (*++p && *p != '*' && !CTYPE(isalpha, *p))
 		continue;
-	    if (*p != 's' || strchr(p, '%') != NULL)
+	    if (*p != 's')
 		return "bad sprintf format for field 4";
 	}
 	if (sp->FNLwantsnames
@@ -377,7 +421,8 @@ SITEparseone(Entry, sp, subbed)
     /* Now tell the groups this site gets that they should feed this site. */
     if (sp != &ME && subbed) {
 	isp = sp - Sites;
-	for (p = subbed, ngp = Groups, i = nGroups; --i >= 0; ngp++)
+	for (p = subbed, u = poison, ngp = Groups, i = nGroups;
+		--i >= 0; ngp++) {
 	    if (*p++) {
 		for (j = 0; j < ngp->nSites; j++)
 		    if (ngp->Sites[j] == NOSITE) {
@@ -387,6 +432,16 @@ SITEparseone(Entry, sp, subbed)
 		if (j == ngp->nSites)
 		    ngp->Sites[ngp->nSites++] = isp;
 	    }
+	    if (*u++) {
+		for (j = 0; j < ngp->nPoison; j++)
+		    if (ngp->Poison[j] == NOSITE) {
+			ngp->Poison[j] = isp;
+			break;
+		    }
+		if (j == ngp->nPoison)
+		    ngp->Poison[ngp->nPoison++] = isp;
+	    }
+        }
     }
 
     /* If this is a duplicate name, find the master. */
@@ -425,7 +480,7 @@ SITEfunnelpatch()
 
     /* Loop over all funnel feeds. */
     for (result = TRUE, i = nSites, sp = Sites; --i >= 0; sp++) {
-	if (sp->Type != FTfunnel)
+	if (sp->Name == NULL || sp->Type != FTfunnel)
 	    continue;
 
 	/* Find the entry they feed in to, give that entry a buffer. */
@@ -474,6 +529,7 @@ SITEparsefile(StartSite)
     char		**strings;
     SITE		*sp;
     char		*subbed;
+    char		*poison;
     STRING		error;
     int			errors;
 
@@ -494,21 +550,24 @@ SITEparsefile(StartSite)
     for (sp = Sites, i = 0; i < nSites; i++, sp++) {
 	sp->Name = NULL;
 	sp->Buffer.Data = NULL;
+        sp->Buffer.Size = sp->Buffer.Used = sp->Buffer.Left = 0 ;
     }
 
     /* Set up scratch subscription list. */
     subbed = NEW(char, nGroups);
+    poison = NEW(char, nGroups);
 
     for (sp = Sites, errors = 0, i = 0; i < nSites; i++) {
 	p = strings[i];
-	if (p[0] == 'M' && p[1] == 'E' && p[2] == NF_FIELD_SEP) {
-	    if ((error = SITEparseone(p, &ME, subbed)) != NULL) {
+	if (p[0] == 'M' && p[1] == 'E' &&
+            ((p[2] == NF_FIELD_SEP) || (p[2] == NF_SUBFIELD_SEP))) {
+	    if ((error = SITEparseone(p, &ME, subbed, poison)) != NULL) {
 		syslog(L_FATAL, "%s bad_newsfeeds %s", MaxLength(p, p), error);
 		errors++;
 	    }
 	    continue;
 	}
-	if ((error = SITEparseone(p, sp, subbed)) != NULL) {
+	if ((error = SITEparseone(p, sp, subbed, poison)) != NULL) {
 	    syslog(L_FATAL, "%s bad_newsfeeds %s", MaxLength(p, p), error);
 	    errors++;
 	    continue;
@@ -531,10 +590,10 @@ SITEparsefile(StartSite)
     /* Free our scratch array, set up the funnel links. */
     nSites = sp - Sites;
     DISPOSE(subbed);
+    DISPOSE(poison);
     DISPOSE(strings);
     if (!SITEfunnelpatch()) {
 	JustCleanup();
 	exit(1);
     }
-    SITElinkall();
 }
