@@ -1,19 +1,25 @@
-/*  $Revision: 1.1 $
+/*  $Revision: 1.42 $
 **
 **  Routines for the remote connect channel.  Create an Internet stream socket
 **  that processes connect to.  If the incoming site is not one of our feeds,
 **  then we pass the connection off to the standard nntp daemon.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 #include <netdb.h>
 
 
-#if	!defined(NETSWAP) && !defined(__bsdi__)
+#if	!defined(NETSWAP)
 #if	!defined(htons)
 extern unsigned short	htons();
 #endif	/* !defined(htons) */
 #if	!defined(htonl)
-extern unsigned long	htonl();
+#if 0   
+extern unsigned long	htonl(); /* nobody should really need this anymore */
+#endif
 #endif	/* !defined(htonl) */
 #endif	/* !defined(NETSWAP) */
 
@@ -27,9 +33,15 @@ typedef struct _REMOTEHOST {
     char	*Name;
     INADDR	Address;
     char	*Password;
+    BOOL	Streaming ;
     char	**Patterns;
 } REMOTEHOST;
 
+typedef struct _REMOTETABLE {
+    INADDR	Address;
+    time_t	Expires;
+} REMOTETABLE;
+
 STATIC INADDR		*RCmaster;
 STATIC int		RCnmaster;
 STATIC char		*RCslaveflag;
@@ -42,6 +54,14 @@ STATIC int		RCnpeerlist;
 STATIC REMOTEHOST	*RCnolimitlist;
 STATIC int		RCnnolimitlist;
 
+/*
+** Stuff needed for limiting incoming connects.
+*/
+STATIC char		RCterm[] = "\r\n";
+STATIC REMOTETABLE	remotetable[REMOTETABLESIZE];
+STATIC int		remotecount;
+STATIC int		remotefirst;
+
 
 /*
 **  See if the site properly entered the password.
@@ -109,6 +129,37 @@ RCismaster(addr)
 }
 
 
+/*
+**  Called when input is ready to read.  Shouldn't happen.
+*/
+/* ARGSUSED0 */
+STATIC FUNCTYPE
+RCrejectreader(cp)
+    CHANNEL	*cp;
+{
+    syslog(L_ERROR, "%s internal RCrejectreader", LogName);
+}
+
+
+/*
+**  Write-done function for rejects.
+*/
+STATIC FUNCTYPE
+RCrejectwritedone(cp)
+    register CHANNEL	*cp;
+{
+    switch (cp->State) {
+    default:
+	syslog(L_ERROR, "%s internal RCrejectwritedone state %d",
+	    CHANname(cp), cp->State);
+	break;
+    case CSwritegoodbye:
+	CHANclose(cp, CHANname(cp));
+	break;
+    }
+}
+
+
 /*
 **  Hand off a descriptor to NNRPD.
 */
@@ -121,6 +172,13 @@ RChandoff(fd, h)
     char	buff[SMBUF];
     int		i;
 
+#if	defined(SOL_SOCKET) && defined(SO_KEEPALIVE)
+    /* Set KEEPALIVE to catch broken socket connections. */
+    i = 1;
+    if (setsockopt(fd, SOL_SOCKET,  SO_KEEPALIVE, (char *)&i, sizeof i) < 0)
+        syslog(L_ERROR, "fd %d cant setsockopt(KEEPALIVE) %m", fd);
+#endif /* defined(SOL_SOCKET) && defined(SO_KEEPALIVE) */
+
     if (SetNonBlocking(fd, FALSE) < 0)
 	syslog(L_ERROR, "%s cant nonblock %d in RChandoff %m", LogName, fd);
     switch (h) {
@@ -164,6 +222,12 @@ RCreader(cp)
     register REMOTEHOST	*rp;
     CHANNEL		*new;
     char		*name;
+    long		reject_val;
+    char		*reject_message;
+    int			count;
+    int			found;
+    time_t		now;
+    CHANNEL		tempchan;
 
     if (cp != RCchan) {
 	syslog(L_ERROR, "%s internal RCreader wrong channel 0x%x not 0x%x",
@@ -178,6 +242,95 @@ RCreader(cp)
 	return;
     }
 
+    /*
+    ** If RemoteTimer is not zero, then check the limits on incoming
+    ** connections on a total and per host basis.
+    **
+    ** The incoming connection table is fixed at 128 entries to make
+    ** calculating the index easy (i + 1) & 7, and to be pretty sure
+    ** that you won't run out of space.  The table is used as a ring
+    ** with new entries being added to the end (wrapping around) and
+    ** expired entries being deleted from the front (again wrapping
+    ** around).  It is doubtful that you will ever use even half of
+    ** the table.
+    **
+    ** There are three parameters controlling the use of the table not
+    ** counting the starting index and count.
+    **
+    ** L = per host incoming connects per X seconds allowed
+    ** T = total incoming connects per X seconds allowed
+    ** X = number of seconds to remember a successful connect
+    **
+    ** First, one pass is made over the live entries deleting any that
+    ** are over X seconds old.  If the entry hasn't expired, compare
+    ** the incoming connection's host address with the entry's host
+    ** address.  If equal, increment the ``found'' counter.
+    **
+    ** Second, if the number of entries now in the table is equal to
+    ** the ``T'' parameter, reject the connection with the ``504''
+    ** error code.
+    **
+    ** 504 Server overloaded, try later
+    **
+    ** Third, if the number of entries now in the table which match
+    ** the incoming connection's host address is equal to the ``L''
+    ** parameter, reject the connection with the ``505'' error code.
+    **
+    ** 505 Connection rejected, you're making too many connects per minute
+    **
+    ** Finally, if neither rejection happened, add the entry to the
+    ** table, and continue on as a normal connect.
+    */
+    tempchan.Address.s_addr = remote.sin_addr.s_addr;
+    reject_message = NULL;
+    if (RemoteTimer != 0) {
+	now = time(NULL);
+	i = remotefirst;
+	count = remotecount;
+	found = 0;
+	while (count--) {
+	    if (remotetable[i].Expires < now) {
+		remotecount--;
+		remotefirst = (remotefirst + 1) & (REMOTETABLESIZE - 1);
+		i = (i + 1) & (REMOTETABLESIZE - 1);
+		continue;
+	    }
+	    if (remotetable[i].Address.s_addr == remote.sin_addr.s_addr)
+		found++;
+	    i = (i + 1) & (REMOTETABLESIZE - 1);
+	}
+	if (remotecount == RemoteTotal) {
+	    reject_val = NNTP_SERVER_TOOBUSY_VAL;
+	    reject_message = NNTP_SERVER_TOOBUSY;
+	}
+	else if (found >= RemoteLimit && !RCnolimit(&tempchan)) {
+	    reject_val = NNTP_TOO_MANY_CONNECTS_VAL;
+	    reject_message = NNTP_TOO_MANY_CONNECTS;
+	}
+	else {
+	    i = (remotefirst + remotecount) & (REMOTETABLESIZE - 1);
+	    remotetable[i].Address = remote.sin_addr;
+	    remotetable[i].Expires = now + RemoteTimer;
+	    remotecount++;
+	}
+    }
+
+    /*
+    ** Create a reject channel to reject the connection.  This is done
+    ** to avoid a call to fork.  
+    */
+    if (reject_message) {
+	new = CHANcreate(fd, CTreject, CSwritegoodbye, RCrejectreader,
+	    RCrejectwritedone);
+	new->Address.s_addr = remote.sin_addr.s_addr;
+	new->Rejected = reject_val;
+	RCHANremove(new);
+	WCHANset(new, reject_message, (int)strlen(reject_message));
+	WCHANappend(new, RCterm, STRLEN(RCterm));
+	WCHANadd(new);
+	return;
+    }
+
     /* See if it's one of our servers. */
     for (name = NULL, rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++)
 	/* SUPPRESS 112 *//* Retrieving long where char is stored */
@@ -187,11 +340,12 @@ RCreader(cp)
 	}
 
     /* If not a server, and not allowing anyone, hand him off. */
-    if (i >= 0)
-	new = NCcreate(fd, rp->Password[0] != '\0');
-    else if (AnyIncoming)
-	new = NCcreate(fd, FALSE);
-    else {
+    if (i >= 0) {
+	new = NCcreate(fd, rp->Password[0] != '\0', FALSE);
+        new->Streaming = rp->Streaming ;
+    } else if (AnyIncoming) {
+	new = NCcreate(fd, FALSE, FALSE);
+    } else {
 	RChandoff(fd, HOnntpd);
 	if (close(fd) < 0)
 	    syslog(L_ERROR, "%s cant close %d %m", LogName, fd);
@@ -200,8 +354,9 @@ RCreader(cp)
 
     /* SUPPRESS 112 *//* Retrieving long where char is stored */
     new->Address.s_addr = remote.sin_addr.s_addr;
-    syslog(L_NOTICE, "%s connected %d",
-	name ? name : inet_ntoa(new->Address), new->fd);
+    syslog(L_NOTICE, "%s connected %d streaming %s",
+           name ? name : inet_ntoa(new->Address), new->fd,
+           (!StreamingOff || new->Streaming) ? "allowed" : "not allowed");
 }
 
 
@@ -237,6 +392,7 @@ RCreadfile(list, count, filename)
     register int	i;
     register REMOTEHOST	*rp;
     register int	j;
+    int			k ;
     char		*pass;
     char		*pats;
     int			errors;
@@ -246,8 +402,10 @@ RCreadfile(list, count, filename)
 	for (rp = *list, i = *count; --i >= 0; rp++) {
 	    DISPOSE(rp->Name);
 	    DISPOSE(rp->Password);
-	    if (rp->Patterns)
+	    if (rp->Patterns) {
+		DISPOSE(rp->Patterns[0]);
 		DISPOSE(rp->Patterns);
+	    }
 	}
 	DISPOSE(*list);
 	*list = NULL;
@@ -294,11 +452,27 @@ RCreadfile(list, count, filename)
 	    pats = NULL;
 	}
 
+        /* Check if the host name ends with '/s' which means that streaming is
+           specifically permitted (as opposed to defaulted). The default
+           for the global StreamingOff is FALSE, meaning any host can use
+           streaming commands. If any entry is hosts.nntp has a suffix of
+           '/s' then StreamingOff is set to TRUE, and then only those
+           hosts.nntp entries with '/s' can use streaming commands. */
+        rp->Streaming = FALSE;
+        if ((k = strlen(buff)) > 2) {
+            if (buff[k - 1] == 's' && buff[k - 2] == '/') {
+                buff[k - 2] = '\0';
+                rp->Streaming = TRUE;
+                StreamingOff = TRUE ;
+            }
+        }
+
 	/* Was host specified as as dotted quad? */
-	if ((rp->Address.s_addr = inet_addr(buff)) != (unsigned long)-1) {
+	if ((rp->Address.s_addr = inet_addr(buff)) != (unsigned int) -1) {
+ 	  syslog(LOG_NOTICE, "think it's a dotquad: %s",buff);
 	    rp->Name = COPY(buff);
 	    rp->Password = COPY(pass);
-	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
+	    rp->Patterns = (pats && *pats) ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	    continue;
 	}
@@ -324,7 +498,7 @@ RCreadfile(list, count, filename)
 	    COPYADDR(&rp->Address, hp->h_addr_list[0]);
 	    rp->Name = COPY(hp->h_name);
 	    rp->Password = COPY(pass);
-	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
+	    rp->Patterns = (pats && *pats) ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	    continue;
 	}
@@ -340,7 +514,7 @@ RCreadfile(list, count, filename)
 	    COPYADDR(&rp->Address, hp->h_addr_list[i]);
 	    rp->Name = COPY(hp->h_name);
 	    rp->Password = COPY(pass);
-	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
+	    rp->Patterns = (pats && *pats) ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	}
 #else
@@ -348,7 +522,7 @@ RCreadfile(list, count, filename)
 	COPYADDR(&rp->Address, hp->h_addr);
 	rp->Name = COPY(hp->h_name);
 	rp->Password = COPY(pass);
-	rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
+	rp->Patterns = (pats && *pats) ? CommaSplit(COPY(pats)) : NULL;
 	rp++;
 #endif	/* defined(h_addr) */
     }
@@ -369,6 +543,7 @@ RCreadlist()
     char	name[sizeof _PATH_INNDHOSTS + sizeof ".nolimit"];
     struct stat	Sb;
 
+    StreamingOff = FALSE ;
     RCreadfile(&RCpeerlist, &RCnpeerlist, INNDHOSTS);
     FileGlue(name, INNDHOSTS, '.', "nolimit");
     if (stat(name, &Sb) >= 0)
@@ -422,7 +597,7 @@ RCcanpost(cp, group)
 	    subvalue = *pat != SUB_NEGATE;
 	    if (!subvalue)
 		pat++;
-	    if (wildmat(group, pat))
+	    if ((match != subvalue) && wildmat(group, pat))
 		match = subvalue;
 	}
 	return match;
@@ -485,7 +660,7 @@ RCsetup(i, master)
     AmSlave = master != NULL;
     if (AmSlave) {
 	/* Dotted quad? */
-	if ((a.s_addr = inet_addr(master)) != (unsigned long)-1) {
+	if ((a.s_addr = inet_addr(master)) != (unsigned int) -1) {
 	    RCnmaster = 1;
 	    RCmaster = NEW(INADDR, 1);
 	    COPYADDR(&RCmaster[0], &a);
