@@ -1,33 +1,18 @@
-/*  $Revision: 1.32 $
+/*  $Revision: 1.40 $
 **
 **  Routines to implement site-feeding.  Mainly working with channels to
 **  do buffering and determine what to send.
 */
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
 
 
-#define SITEmovetohead(sp_) \
-    if (SITEhead != sp_) { \
-	if (SITEtail == sp_) { \
-	    SITEtail = sp_->Prev; \
-	    SITEtail->Next = NULL; \
-	} \
-	else { \
-	    if (sp_->Prev) \
-		sp_->Prev->Next = sp_->Next; \
-	    if (sp_->Next) \
-		sp_->Next->Prev = sp_->Prev; \
-	} \
-	sp_->Prev = NULL; \
-	sp_->Next = SITEhead->Next; \
-	SITEhead->Prev = sp_; \
-	SITEhead = sp_; \
-    } \
-    else
-    
 STATIC int	SITEcount;
-STATIC SITE	*SITEhead;
-STATIC SITE	*SITEtail;
+STATIC int	SITEhead = NOSITE;
+STATIC int	SITEtail = NOSITE;
 STATIC char	SITEshell[] = _PATH_SH;
 
 
@@ -57,23 +42,42 @@ SITEwritedone(cp)
 /*
 **  Make a site start spooling.
 */
-STATIC BOOL
-SITEspool(sp)
+BOOL
+SITEspool(sp, cp)
     register SITE	*sp;
+    CHANNEL		*cp;
 {
     int			i;
+    char		buff[SPOOLNAMEBUFF];
+    char		*name;
 
-    sp->Spooling = TRUE;
-    sp->Process = -1;
-    i = open(sp->SpoolName, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+    name = sp->SpoolName;
+    i = open(name, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+    if (i < 0 && errno == EISDIR) {
+	FileGlue(buff, sp->SpoolName, '/', "togo");
+	name = buff;
+	i = open(buff, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+    }
     if (i < 0) {
-	IOError("site batch file");
-	syslog(L_ERROR, "%s cant open %s %m", sp->Name, sp->SpoolName);
+	i = errno;
+	syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
+	IOError("site batch file", i);
 	sp->Channel = NULL;
 	return FALSE;
     }
     if (AmRoot)
-	xchown(sp->SpoolName);
+	xchown(name);
+    if (cp) {
+      if (cp->fd >= 0)
+        syslog(L_ERROR, "DEBUG ERROR SITEspool trashed:%d %s:%d", cp->fd, sp->Name, i);
+	/* CPU-eating bug, killed by kre. */
+	WCHANremove(cp);
+	RCHANremove(cp);
+	SCHANremove(cp);
+	(void)close(cp->fd);
+	cp->fd = i;
+	return TRUE;
+    }
     sp->Channel = CHANcreate(i, CTfile, CSwriting, SITEreader, SITEwritedone);
     if (sp->Channel == NULL) {
 	syslog(L_ERROR, "%s cant channel %m", sp->Name);
@@ -81,10 +85,37 @@ SITEspool(sp)
 	return FALSE;
     }
     WCHANset(sp->Channel, "", 0);
+    sp->Spooling = TRUE;
     return TRUE;
 }
 
 
+/*
+**  Delete a site from the file writing list.  Can be called even if
+**  site is not on the list.
+*/
+static void
+SITEunlink(sp)
+    SITE		*sp;
+{
+    if (sp->Next != NOSITE || sp->Prev != NOSITE
+     || (SITEhead != NOSITE && sp == &Sites[SITEhead]))
+	SITEcount--;
+
+    if (sp->Next != NOSITE)
+	Sites[sp->Next].Prev = sp->Prev;
+    else if (SITEtail != NOSITE && sp == &Sites[SITEtail])
+	SITEtail = sp->Prev;
+
+    if (sp->Prev != NOSITE)
+	Sites[sp->Prev].Next = sp->Next;
+    else if (SITEhead != NOSITE && sp == &Sites[SITEhead])
+	SITEhead = sp->Next;
+
+    sp->Next = sp->Prev = NOSITE;
+}
+
+
 /*
 **  Find the oldest "file feed" site and buffer it.
 */
@@ -95,12 +126,25 @@ SITEbufferoldest()
     register BUFFER	*bp;
     register BUFFER	*out;
 
-    /* Go backwards and find the oldest file. */
-    for (sp = SITEtail; sp; sp = sp->Prev)
-	if (sp->Type == FTfile)
-	    break;
-    if (sp == NULL) {
+    /* Get the oldest user of a file. */
+    if (SITEtail == NOSITE) {
 	syslog(L_ERROR, "%s internal no oldest site found", LogName);
+	SITEcount = 0;
+	return;
+    }
+
+    sp = &Sites[SITEtail];
+    SITEunlink(sp);
+
+    if (sp->Buffered) {
+	syslog(L_ERROR, "%s internal oldest (%s) was buffered", LogName,
+	    sp->Name);
+	return;
+    }
+
+    if (sp->Type != FTfile) {
+	syslog(L_ERROR, "%s internal oldest (%s) not FTfile", LogName,
+	    sp->Name);
 	return;
     }
 
@@ -123,14 +167,61 @@ SITEbufferoldest()
 
     /* If there's any unwritten data, copy it. */
     out = &sp->Channel->Out;
-    if (out->Left)
+    if (out->Left) {
 	BUFFset(bp, &out->Data[out->Used], out->Left);
+	out->Left = 0;
+    }
 
     /* Now close the original channel. */
     CHANclose(sp->Channel, sp->Name);
-    SITEcount--;
+    sp->Channel = NULL;
 }
 
+/*
+ * *  Bilge Site's Channel out buffer.
+ */
+STATIC          BOOL
+SITECHANbilge(sp)
+    register SITE  *sp;
+{
+    int             fd, i;
+    char            buff[SPOOLNAMEBUFF];
+    char           *name;
+
+    name = sp->SpoolName;
+    fd = open(name, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+    if (fd < 0 && errno == EISDIR) {
+        FileGlue(buff, sp->SpoolName, '/', "togo");
+        name = buff;
+        fd = open(buff, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
+    }
+    if (fd < 0) {
+	int oerrno = errno ;
+        syslog(L_ERROR, "%s cant open %s %m", sp->Name, name);
+        IOError("site batch file",oerrno);
+        sp->Channel = NULL;
+        return FALSE;
+    }
+    while(sp->Channel->Out.Left > 0) {
+        i = write(fd, (POINTER) &sp->Channel->Out.Data[sp->Channel->Out.Used],
+            (SIZE_T) sp->Channel->Out.Left);
+        if(i <= 0) {
+            syslog(L_ERROR,"%s cant spool count %d",CHANname(sp->Channel),
+                sp->Channel->Out.Left);
+            close(fd);
+            return FALSE;
+        }
+        sp->Channel->Out.Left -= i;
+        sp->Channel->Out.Used += i;
+    }
+    close(fd);
+    DISPOSE(sp->Channel->Out.Data);
+    sp->Channel->Out.Data = NEW(char, SMBUF);
+    sp->Channel->Out.Size = SMBUF;
+    sp->Channel->Out.Left = 0;
+    sp->Channel->Out.Used = 0;
+    return TRUE;
+}
 
 /*
 **  Check if we need to write out the site's buffer.  If we're buffered
@@ -146,9 +237,9 @@ SITEflushcheck(sp, bp)
 
     /* If we're buffered, and we hit the flushpoint, do an LRU. */
     if (sp->Buffered) {
-	if (bp->Used < sp->Flushpoint)
+	if (bp->Left < sp->Flushpoint)
 	    return;
-	if (SITEcount > MaxOutgoing)
+	while (SITEcount >= MaxOutgoing)
 	    SITEbufferoldest();
 	if (!SITEsetup(sp) || sp->Buffered) {
 	    syslog(L_ERROR, "%s cant unbuffer %m", sp->Name);
@@ -156,6 +247,8 @@ SITEflushcheck(sp, bp)
 	}
 	WCHANsetfrombuffer(sp->Channel, bp);
 	WCHANadd(sp->Channel);
+	/* Reset buffer; data has been moved. */
+	BUFFset(bp, "", 0);
     }
 
     if (PROCneedscan)
@@ -170,18 +263,17 @@ SITEflushcheck(sp, bp)
      && !CHANsleeping(cp))
 	WCHANadd(cp);
 
+    cp->LastActive = Now.time;
+
     /* If we're a channel that's getting big, see if we need to spool. */
     if (sp->Type == FTfile || sp->StartSpooling == 0 || i < sp->StartSpooling)
 	return;
 
-    if (!SITEspool(sp)) {
+    syslog(L_ERROR, "%s spooling %d bytes", sp->Name, i);
+    if(!SITECHANbilge(sp)) {
 	syslog(L_ERROR, "%s overflow %d bytes", sp->Name, i);
 	return;
     }
-    syslog(L_ERROR, "%s spooling %d bytes", sp->Name, i);
-    WCHANsetfrombuffer(sp->Channel, &cp->Out);
-    WCHANadd(sp->Channel);
-    CHANclose(cp, CHANname(cp));
 }
 
 
@@ -196,11 +288,19 @@ SITEwrite(sp, text)
     static char		PREFIX[] = { EXP_CONTROL, '\0' };
     register BUFFER	*bp;
 
-    bp = sp->Buffered ? &sp->Buffer : &sp->Channel->Out;
+    if (sp->Buffered)
+	bp = &sp->Buffer;
+    else {
+	if (sp->Channel == NULL)
+	    return;
+	sp->Channel->LastActive = Now.time;
+	bp = &sp->Channel->Out;
+    }
     BUFFappend(bp, PREFIX, STRLEN(PREFIX));
     BUFFappend(bp, text, (int)strlen(text));
     BUFFappend(bp, "\n", 1);
-    WCHANadd(sp->Channel);
+    if (sp->Channel != NULL)
+	WCHANadd(sp->Channel);
 }
 
 
@@ -213,17 +313,28 @@ SITEwritefromflags(sp, Data)
     ARTDATA		*Data;
 {
     static char		ITEMSEP[] = " ";
+    static char		NL[] = "\n";
+    char		pbuff[12];
     register char	*p;
     register BOOL	Dirty;
     register BUFFER	*bp;
     register SITE	*spx;
     register int	i;
 
-    bp = sp->Buffered ? &sp->Buffer : &sp->Channel->Out;
+    if (sp->Buffered)
+	bp = &sp->Buffer;
+    else {
+	/* This should not happen, but if we tried to spool and failed,
+	 * e.g., because of a bad F param for this site, we can get
+	 * into this state.  We already logged a message so give up. */
+	if (sp->Channel == NULL)
+	    return;
+	bp = &sp->Channel->Out;
+    }
     for (Dirty = FALSE, p = sp->FileFlags; *p; p++) {
 	switch (*p) {
 	default:
-	    syslog(L_ERROR, "%s internal SITEwritefromflags %c", sp->Name, p);
+	    syslog(L_ERROR, "%s internal SITEwritefromflags %c", sp->Name, *p);
 	    continue;
 	case FEED_BYTESIZE:
 	    if (Dirty)
@@ -247,11 +358,32 @@ SITEwritefromflags(sp, Data)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
 	    BUFFappend(bp, Data->Newsgroups, Data->NewsgroupsLength);
 	    break;
+	case FEED_HEADERS:
+	    if (Dirty)
+		BUFFappend(bp, NL, STRLEN(NL));
+	    BUFFappend(bp, Data->Headers->Data, Data->Headers->Left);
+	    break;
+	case FEED_OVERVIEW:
+	    if (Dirty)
+		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+	    BUFFappend(bp, Data->Overview->Data, Data->Overview->Left);
+	    break;
 	case FEED_REPLIC:
 	    if (Dirty)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
 	    BUFFappend(bp, Data->Replic, Data->ReplicLength);
 	    break;
+	case FEED_TIMERECEIVED:
+	    if (Dirty)
+		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+	    BUFFappend(bp, Data->TimeReceived, Data->TimeReceivedLength);
+	    break;
+	case FEED_TIMEPOSTED:
+	    if (Dirty)
+		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
+	    sprintf(pbuff, "%ld", Data->Posted);
+	    BUFFappend(bp, pbuff, strlen(pbuff));
+	    break;
 	case FEED_MESSAGEID:
 	    if (Dirty)
 		BUFFappend(bp, ITEMSEP, STRLEN(ITEMSEP));
@@ -322,13 +454,12 @@ SITEsend(sp, Data)
     default:
 	syslog(L_ERROR, "%s internal SITEsend type %d", sp->Name, sp->Type);
 	break;
+    case FTlogonly:
+	break;
     case FTfunnel:
 	syslog(L_ERROR, "%s funnel_send", sp->Name);
 	break;
     case FTfile:
-	if (SITEcount > MaxOutgoing)
-	    SITEmovetohead(sp);
-	/* FALLTHROUGH */
     case FTchannel:
     case FTexploder:
 	SITEwritefromflags(sp, Data);
@@ -387,16 +518,16 @@ SITEsend(sp, Data)
 **  a file.  Flush and restart.
 */
 STATIC FUNCTYPE
-SITEspoolwake(cp, Argument)
+SITEspoolwake(cp)
     CHANNEL	*cp;
-    char	*Argument;
 {
     SITE	*sp;
     int		*ip;
 
-    ip = CAST(int*, Argument);
+    ip = CAST(int*, cp->Argument);
     sp = &Sites[*ip];
-    DISPOSE(Argument);
+    DISPOSE(cp->Argument);
+    cp->Argument = NULL;
     if (sp->Channel != cp) {
 	syslog(L_ERROR, "%s internal SITEspoolwake %s got %d, not %d",
 	    LogName, sp->Name, cp->fd, sp->Channel->fd);
@@ -415,7 +546,7 @@ STATIC BOOL
 SITEstartprocess(sp)
     SITE		*sp;
 {
-    register int	i;
+    register PID_T	i;
     STRING		argv[MAX_BUILTIN_ARGV];
     char		*process;
     int			*ip;
@@ -426,7 +557,7 @@ SITEstartprocess(sp)
 	syslog(L_ERROR, "%s cant pipe %m", sp->Name);
 	return FALSE;
     }
-    CloseOnExec(pan[PIPE_WRITE]);
+    CloseOnExec(pan[PIPE_WRITE], TRUE);
 
     /* Set up the argument vector. */
     process = COPY(sp->Param);
@@ -450,13 +581,13 @@ SITEstartprocess(sp)
 	DISPOSE(process);
 	return TRUE;
     }
-    DISPOSE(process);
 
     /* Error.  Switch to spooling. */
-    syslog(L_ERROR, "%s spooling", sp->Name);
+    syslog(L_ERROR, "%s cant spawn spooling %m", sp->Name);
     (void)close(pan[PIPE_WRITE]);
     (void)close(pan[PIPE_READ]);
-    if (!SITEspool(sp))
+    DISPOSE(process);
+    if (!SITEspool(sp, (CHANNEL *)NULL))
 	return FALSE;
 
     /* We'll try to restart the channel later. */
@@ -464,7 +595,7 @@ SITEstartprocess(sp)
     ip = NEW(int, 1);
     *ip = sp - Sites;
     SCHANadd(sp->Channel, (time_t)(Now.time + CHANNEL_RETRY_TIME),
-	SITEspoolwake, (char *)ip);
+	(POINTER)NULL, SITEspoolwake, (POINTER)ip);
     return TRUE;
 }
 
@@ -478,14 +609,16 @@ SITEbuffer(sp)
 {
     register BUFFER	*bp;
 
+    SITEunlink(sp);
     sp->Buffered = TRUE;
     sp->Channel = NULL;
     bp = &sp->Buffer;
     if (bp->Size == 0) {
 	bp->Size = sp->Flushpoint;
 	bp->Data = NEW(char, bp->Size);
+	BUFFset(bp, "", 0);
     }
-    else {
+    else if (bp->Size < sp->Flushpoint) {
 	bp->Size = sp->Flushpoint;
 	RENEW(bp->Data, char, bp->Size);
     }
@@ -494,6 +627,30 @@ SITEbuffer(sp)
 }
 
 
+/*
+**  Link a site at the head of the "currently writing to a file" list
+*/
+static void
+SITEmovetohead(sp)
+    SITE		*sp;
+{
+    if (SITEhead == NOSITE
+     && (sp->Next != NOSITE || sp->Prev != NOSITE
+      || sp == &Sites[SITEhead] || sp == &Sites[SITEtail]))
+	SITEunlink(sp);
+
+    if ((sp->Next = SITEhead) != NOSITE)
+	Sites[SITEhead].Prev = sp - Sites;
+    sp->Prev = NOSITE;
+
+    SITEhead = sp - Sites;
+    if (SITEtail == NOSITE)
+	SITEtail = sp - Sites;
+
+    SITEcount++;
+}
+
+
 /*
 **  Set up a site's feed.  This means opening a file or channel if needed.
 */
@@ -502,6 +659,7 @@ SITEsetup(sp)
     register SITE	*sp;
 {
     int			fd;
+    int			oerrno;
 
     switch (sp->Type) {
     default:
@@ -509,25 +667,28 @@ SITEsetup(sp)
 	    sp->Name, sp->Type);
 	return FALSE;
     case FTfunnel:
+    case FTlogonly:
     case FTprogram:
 	/* Nothing to do here. */
 	break;
     case FTfile:
-	SITEcount++;
-	if (SITEcount > MaxOutgoing)
+	if (SITEcount >= MaxOutgoing)
 	    SITEbuffer(sp);
 	else {
 	    sp->Buffered = FALSE;
 	    fd = open(sp->Param, O_APPEND | O_CREAT | O_WRONLY, BATCHFILE_MODE);
 	    if (fd < 0) {
 		if (errno == EMFILE) {
+		    syslog(L_ERROR, "%s cant open %s %m", sp->Name, sp->Param);
 		    SITEbuffer(sp);
 		    break;
 		}
-		IOError("site file");
+		oerrno = errno;
 		syslog(L_NOTICE, "%s cant open %s %m", sp->Name, sp->Param);
+		IOError("site file", oerrno);
 		return FALSE;
 	    }
+	    SITEmovetohead(sp);
 	    if (AmRoot)
 		xchown(sp->Param);
 	    sp->Channel = CHANcreate(fd, CTfile, CSwriting,
@@ -560,7 +721,7 @@ SITEprocdied(sp, process, pp)
 {
     syslog(pp->Status ? L_ERROR : L_NOTICE, "%s exit %d elapsed %ld pid %ld",
 	sp->Name ? sp->Name : "?", pp->Status,
-	(pp->Collected - pp->Started) / 60L, (long)pp->Pid);
+	pp->Collected - pp->Started, (long)pp->Pid);
     if (sp->Process != process || sp->Name == NULL)
 	/* We already started a new process for this channel
 	 * or this site has been dropped. */
@@ -591,8 +752,8 @@ SITEchanclose(cp)
 	    /* Found the site that has this channel.  Start that
 	     * site spooling, copy any data that might be pending,
 	     * and arrange to retry later. */
-	    if (!SITEspool(sp)) {
-		syslog(L_ERROR, "%s loss %d bytes", sp->Name, i);
+	    if (!SITEspool(sp, (CHANNEL *)NULL)) {
+		syslog(L_ERROR, "%s loss %d bytes", sp->Name, cp->Out.Left);
 		return;
 	    }
 	    WCHANsetfrombuffer(sp->Channel, &cp->Out);
@@ -600,7 +761,7 @@ SITEchanclose(cp)
 	    ip = NEW(int, 1);
 	    *ip = sp - Sites;
 	    SCHANadd(sp->Channel, (time_t)(Now.time + CHANNEL_RETRY_TIME),
-		SITEspoolwake, (char *)ip);
+		(POINTER)NULL, SITEspoolwake, (POINTER)ip);
 	    break;
 	}
 }
@@ -615,12 +776,21 @@ SITEflush(sp, Restart)
     BOOL		Restart;
 {
     register CHANNEL	*cp;
+    register BUFFER	*out;
+    int			count;
+
+    if (sp->Name == NULL)
+	return;
+
+    if (Restart)
+	SITEforward(sp, "flush");
 
     switch (sp->Type) {
     default:
 	syslog(L_ERROR, "%s internal SITEflush %d", sp->Name, sp->Type);
 	return;
 
+    case FTlogonly:
     case FTprogram:
     case FTfunnel:
 	/* Nothing to do here. */
@@ -628,7 +798,7 @@ SITEflush(sp, Restart)
 
     case FTchannel:
     case FTexploder:
-	/* If spooling, close the file right now. */
+	/* If spooling, close the file right now -- documented behavior. */
 	if (sp->Spooling && (cp = sp->Channel) != NULL) {
 	    (void)WCHANflush(cp);
 	    CHANclose(cp, CHANname(cp));
@@ -637,29 +807,61 @@ SITEflush(sp, Restart)
 	break;
 
     case FTfile:
+	/* We must ensure we have a file open for this site, so if
+	 * we're buffered we HACK and pretend we have no sites
+	 * for a moment. */
+	if (sp->Buffered) {
+	    count = SITEcount;
+	    SITEcount = 0;
+	    if (!SITEsetup(sp) || sp->Buffered)
+		syslog(L_ERROR, "%s cant unbuffer to flush", sp->Name);
+	    else
+		BUFFswap(&sp->Buffer, &sp->Channel->Out);
+	    SITEcount += count;
+	}
 	break;
     }
 
     /* We're only dealing with files and channels now. */
     if ((cp = sp->Channel) != NULL)
 	(void)WCHANflush(cp);
-    sp->Channel = NULL;
 
     /* Restart the site, copy any pending data. */
     if (Restart) {
 	if (!SITEsetup(sp))
 	    syslog(L_ERROR, "%s cant restart %m", sp->Name);
-	else if (cp != NULL)
-	    WCHANsetfrombuffer(sp->Channel, &cp->Out);
+	else if (cp != NULL) {
+	    if (sp->Buffered) {
+		/* SITEsetup had to buffer us; save any residue. */
+		out = &cp->Out;
+	        if (out->Left)
+		    BUFFset(&sp->Buffer, &out->Data[out->Used], out->Left);
+	    }
+	    else
+		WCHANsetfrombuffer(sp->Channel, &cp->Out);
+	}
+    }
+    else if (cp != NULL && cp->Out.Left) {
+	if (sp->Type == FTfile || sp->Spooling) {
+	    /* Can't flush a file?  Hopeless. */
+	    syslog(L_ERROR, "%s dataloss %d", sp->Name, cp->Out.Left);
+	    return;
+	}
+	/* Must be a working channel; spool and retry. */
+	syslog(L_ERROR, "%s spooling %d bytes", sp->Name, cp->Out.Left);
+	if (SITEspool(sp, cp))
+	    SITEflush(sp, FALSE);
+	return;
     }
-    else if (cp != NULL && cp->Out.Left)
-	syslog(L_ERROR, "%s dataloss %d", sp->Name, cp->Out.Left);
 
     /* Close the old channel if it was open. */
     if (cp != NULL) {
+        /* Make sure we have no dangling pointers to it. */
+	if (!Restart)
+	    sp->Channel = NULL;
 	CHANclose(cp, sp->Name);
 	if (sp->Type == FTfile)
-	    SITEcount--;
+	    SITEunlink(sp);
     }
 }
 
@@ -690,21 +892,66 @@ SITEwantsgroup(sp, name)
     register char	*name;
 {
     register BOOL	match;
+    register BOOL	subvalue;
     register char	*pat;
     register char	**argv;
-    register BOOL	subvalue;
 
-    for (match = SUB_DEFAULT, argv = sp->Patterns; (pat = *argv++) != NULL; ) {
-	subvalue = *pat != SUB_NEGATE;
+    match = SUB_DEFAULT;
+    if (ME.Patterns) {
+	for (argv = ME.Patterns; (pat = *argv++) != NULL; ) {
+	    subvalue = *pat != SUB_NEGATE && *pat != SUB_POISON;
+	    if (!subvalue)
+		pat++;
+	    if ((match != subvalue) && wildmat(name, pat))
+		match = subvalue;
+	}
+    }
+    for (argv = sp->Patterns; (pat = *argv++) != NULL; ) {
+	subvalue = *pat != SUB_NEGATE && *pat != SUB_POISON;
 	if (!subvalue)
 	    pat++;
-	if (wildmat(name, pat))
+	if ((match != subvalue) && wildmat(name, pat))
 	    match = subvalue;
     }
     return match;
 }
 
 
+/*
+**  Run down the site's pattern list and see if specified newsgroup is
+**  considered poison.
+*/
+BOOL
+SITEpoisongroup(sp, name)
+    register SITE	*sp;
+    register char	*name;
+{
+    register BOOL	match;
+    register BOOL	poisonvalue;
+    register char	*pat;
+    register char	**argv;
+
+    match = SUB_DEFAULT;
+    if (ME.Patterns) {
+	for (argv = ME.Patterns; (pat = *argv++) != NULL; ) {
+	    poisonvalue = *pat == SUB_POISON;
+	    if (*pat == SUB_NEGATE || *pat == SUB_POISON)
+		pat++;
+	    if (wildmat(name, pat))
+		match = poisonvalue;
+	}
+    }
+    for (argv = sp->Patterns; (pat = *argv++) != NULL; ) {
+	poisonvalue = *pat == SUB_POISON;
+	if (*pat == SUB_NEGATE || *pat == SUB_POISON)
+	    pat++;
+	if (wildmat(name, pat))
+	    match = poisonvalue;
+    }
+    return match;
+}
+
+
 /*
 **  Find a site.
 */
@@ -754,6 +1001,10 @@ SITEfree(sp)
 	CHANclose(sp->Channel, CHANname(sp->Channel));
 	sp->Channel = NULL;
     }
+
+    SITEunlink(sp);
+
+    sp->Name = NULL;
     if (sp->Process > 0) {
 	/* Kill the backpointer so PROCdied won't call us. */
 	PROCunwatch(sp->Process);
@@ -762,7 +1013,6 @@ SITEfree(sp)
     if (sp->Entry) {
 	DISPOSE(sp->Entry);
 	sp->Entry = NULL;
-	sp->Name = NULL;
     }
     if (sp->Param) {
 	DISPOSE(sp->Param);
@@ -789,6 +1039,11 @@ SITEfree(sp)
 	sp->Buffer.Data = NULL;
 	sp->Buffer.Size = 0;
     }
+    if (sp->FNLnames.Data) {
+	DISPOSE(sp->FNLnames.Data);
+	sp->FNLnames.Data = NULL;
+	sp->FNLnames.Size = 0;
+    }
 
     /* If this site was a master, find a new one. */
     if (sp->IsMaster) {
@@ -807,38 +1062,167 @@ SITEfree(sp)
 
 
 /*
-**  Drop a site.
+**  If a site is an exploder or funnels into one, forward a command
+**  to it.
 */
 void
-SITEdrop(sp)
-    SITE	*sp;
+SITEforward(sp, text)
+    register SITE	*sp;
+    char		*text;
 {
-    SITEflush(sp, FALSE);
-    SITEfree(sp);
-    sp->Name = NULL;
-    SITElinkall();
+    register SITE	*fsp;
+    register char	*p;
+    char		buff[SMBUF];
+
+    fsp = sp;
+    if (fsp->Funnel != NOSITE)
+	fsp = &Sites[fsp->Funnel];
+    if (sp->Name == NULL || fsp->Name == NULL)
+	return;
+    if (fsp->Type == FTexploder) {
+	(void)strcpy(buff, text);
+	if (fsp != sp && fsp->FNLwantsnames) {
+	    p = buff + strlen(buff);
+	    *p++ = ' ';
+	    (void)strcpy(p, sp->Name);
+	}
+	SITEwrite(fsp, buff);
+    }
 }
 
 
 /*
-**  Put all the feeds into a doubly-linked list.
+**  Drop a site.
 */
 void
-SITElinkall()
+SITEdrop(sp)
+    SITE		*sp;
 {
+    SITEforward(sp, "drop");
+    SITEflush(sp, FALSE);
+    SITEfree(sp);
+}
+
+
+/*
+**  Append info about the current state of the site to the buffer
+*/
+void
+SITEinfo(bp, sp, Verbose)
+    register BUFFER	*bp;
     register SITE	*sp;
+    BOOL		Verbose;
+{
+    static char		FREESITE[] = "<<No name>>\n\n";
+    register char	*p;
+    register CHANNEL	*cp;
+    char		*sep;
+    char		buff[BUFSIZ];
 
-    if (nSites < 2)
+    if (sp->Name == NULL) {
+	BUFFappend(bp, FREESITE, STRLEN(FREESITE));
 	return;
+    }
+
+    p = buff;
+    (void)sprintf(buff, "%s%s:\t", sp->Name, sp->IsMaster ? "(*)" : "");
+    p += strlen(p);
+
+    if (sp->Type == FTfunnel) {
+	sp = &Sites[sp->Funnel];
+	(void)sprintf(p, "funnel -> %s: ", sp->Name);
+	p += strlen(p);
+    }
+
+    switch (sp->Type) {
+    default:
+	(void)sprintf(p, "unknown feed type %d", sp->Type);
+	p += strlen(p);
+	break;
+    case FTerror:
+    case FTfile:
+	p += strlen(strcpy(p, "file"));
+	if (sp->Buffered) {
+	    (void)sprintf(p, " buffered(%ld)", sp->Buffer.Left);
+	    p += strlen(p);
+	}
+	else if ((cp = sp->Channel) == NULL)
+	    p += strlen(strcpy(p, " no channel?"));
+	else {
+	    (void)sprintf(p, " open fd=%d, in mem %ld", cp->fd, cp->Out.Left);
+	    p += strlen(p);
+	}
+	break;
+    case FTchannel:
+	p += strlen(strcpy(p, "channel"));
+	goto Common;
+    case FTexploder:
+	p += strlen(strcpy(p, "exploder"));
+Common:
+	if (sp->Process >= 0) {
+	    (void)sprintf(p, " pid=%ld", (long) sp->pid);
+	    p += strlen(p);
+	}
+	if (sp->Spooling)
+	    p += strlen(strcpy(p, " spooling"));
+	if ((cp = sp->Channel) == NULL)
+	    p += strlen(strcpy(p, " no channel?"));
+	else {
+	    (void)sprintf(p, " fd=%d, in mem %ld", cp->fd, cp->Out.Left);
+	    p += strlen(p);
+	}
+	break;
+    case FTfunnel:
+	p += strlen(strcpy(p, "recursive funnel"));
+	break;
+    case FTlogonly:
+	p += strlen(strcpy(p, "log only"));
+	break;
+    case FTprogram:
+	p += strlen(strcpy(p, "program"));
+	if (sp->FNLwantsnames)
+	    p += strlen(strcpy(p, " with names"));
+	break;
+    }
+    *p++ = '\n';
+    if (Verbose) {
+	sep = "\t";
+	if (sp->Buffered && sp->Flushpoint) {
+	    (void)sprintf(p, "%sFlush @ %ld", sep, sp->Flushpoint);
+	    p += strlen(p);
+	    sep = "; ";
+	}
+	if (sp->StartWriting || sp->StopWriting) {
+	    (void)sprintf(p, "%sWrite [%ld..%ld]", sep,
+		sp->StopWriting, sp->StartWriting);
+	    p += strlen(p);
+	    sep = "; ";
+	}
+	if (sp->StartSpooling) {
+	    (void)sprintf(p, "%sSpool @ %ld", sep, sp->StartSpooling);
+	    p += strlen(p);
+	    sep = "; ";
+	}
+	if (sep[0] != '\t')
+	    *p++ = '\n';
+	if (sp->Spooling && sp->SpoolName) {
+	    (void)sprintf(p, "\tSpooling to \"%s\"\n", sp->SpoolName);
+	    p += strlen(p);
+	}
+	if ((cp = sp->Channel) != NULL) {
+	    (void)sprintf(p, "\tChannel created %.12s",
+		ctime(&cp->Started) + 4);
+	    p += strlen(p);
+	    (void)sprintf(p, ", last active %.12s\n",
+		ctime(&cp->LastActive) + 4);
+	    p += strlen(p);
+	    if (cp->Waketime > Now.time) {
+		(void)sprintf("\tSleeping until %.12s\n",
+		    ctime(&cp->Waketime) + 4);
+		p += strlen(p);
+	    }
+	}
 
-    SITEhead = &Sites[0];
-    SITEtail = &Sites[nSites - 1];
-    for (sp = SITEhead; ++sp < SITEtail; )
-	sp[-1].Next = sp[1].Prev = sp;
-    SITEhead->Prev = NULL;
-    if (SITEhead->Next)
-	SITEhead->Next->Prev = SITEhead;
-    SITEtail->Next = NULL;
-    if (SITEtail->Prev)
-	SITEtail->Prev->Next = SITEtail;
+    }
+    BUFFappend(bp, buff, p - buff);
 }
