@@ -1,4 +1,4 @@
-/*  $Revision: 1.31 $
+/*  $Revision: 1.32 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
@@ -659,7 +659,7 @@ main(argc, argv, env)
 	(void)fflush(stdout);
 	if (ChangeTrace) {
 	    Tracing = Tracing ? FALSE : TRUE;
-	    syslog(L_TRACE, "trace %abled", Tracing ? "en" : "dis");
+	    syslog(L_TRACE, "trace %sabled", Tracing ? "en" : "dis");
 	    ChangeTrace = FALSE;
 	}
 	if (PushedBack) {
