@@ -4,7 +4,7 @@
 # Start Date:   Sat, 24 Aug 1996 23:45:34 +0200
 # Project:      INN
 # File:         innshellvars.tcl
-# RCSId:        $Id: innshellvars.tcl,v 1.10 1996/11/28 12:52:06 brister Exp $
+# RCSId:        $Id: innshellvars.tcl,v 1.11 1996/12/05 00:43:46 brister Exp $
 # Description:  Set up any and all variables that an INN tcl script
 #               might need.
 
@@ -81,6 +81,9 @@ set inn_locks "/var/news/locks"
 ##  =()<set inn_log "@<_PATH_LOGFILE>@">()=
 set inn_log "/var/log/news/news"
 
+##  =()<set inn_mail_badcontrols [ expr { "@<MAIL_BADCONTROLS>@" == "DO" ? 1 : 0 } ]>()=
+set inn_mail_badcontrols [ expr { "DONT" == "DO" ? 1 : 0 } ]
+
 ##  =()<set inn_mailcmd "@<_PATH_MAILCMD>@">()=
 set inn_mailcmd "/usr/bin/Mail"
 
